/*****************************************************************************
 *****************************************************************************
 *     _    __  __ ____  _____ ____             _          _               
 *    / \  |  \/  | __ )| ____|  _ \  __      _(_)_ __ ___| | ___  ___ ___ 
 *   / _ \ | |\/| |  _ \|  _| | |_) | \ \ /\ / / | '__/ _ \ |/ _ \/ __/ __|
 *  / ___ \| |  | | |_) | |___|  _ <   \ V  V /| | | |  __/ |  __/\__ \__ \
 * /_/   \_\_|  |_|____/|_____|_| \_\   \_/\_/ |_|_|  \___|_|\___||___/___/
 *
 *																			 
 * Module Name:																  
 * ============
 *
 *			ERROR_Handle.h
 *
 * Purpose:			
 * ========
 *                      provides error codes 
 *			
 * Author(s) & Creation Date:	
 * ==========================
 *
 *			Timo Koch, 2004/03/01
 *
 *
 * Copyright:	
 * ==========
 *
 *			AMBER wireless GmbH, Inc.
 *			Albin Koebis Strasse 18
 *			D-50858 Koeln, Germany
 *			Phone:		+49 (0) 2203-369472
 *			Fax:		+49 (0) 2203-459883
 *			Internet:	www.amber-wireless.de
 *
 *----------------------------------------------------------------
 * All software and related documentation is provided "AS IS" and 
 * without warranty or support of any kind and AMBER wireless 
 * expressly disclaims all other warranties, express or implied, 
 * including, but not limited to, the implied warranties of 
 * merchantability and fitness for a particular purpose. Under no 
 * circumstances shall AMBER wireless be liable for any 
 * incidental, special or consequential damages that result from 
 * the use or inability to use the software or related 
 * documentation, even if AMBER wireless has been advised of 
 * the liability.
 *
 * Unless otherwise stated, software written and copyrighted by 
 * AMBER wireless is distributed as "freeware". You may use 
 * and modify this software without any charge or restriction. 
 * You may distribute to others, as long as the original author 
 * is acknowledged.
 *
 * You may not use the Program in non-AMBER devices.
 *
 *****************************************************************************
 *****************************************************************************/




//Error codes
#define ERROR_OK                                0x00  //No error
#define ERROR_INVALID_NO_OF_PARAMETERS          0x01  //The number of bytes in the request does not correspond to the protocol specification
#define ERROR_DURATION_OUT_OF_RANGE             0x02  //The given duration value is not valid according to the specification
#define ERROR_INVALID_MODE                      0x03  //The selected mode is not valid according to the specification
#define ERROR_TIMEOUTA                          0x04  //timeout occurred
#define ERROR_UNKNOWN_ERROR                     0x05  //An unknown error occurred
#define ERROR_NAME_TOO_LONG                     0x06  //The number of bytes in the name string is longer than the maximum specified value
#define ERROR_INVALID_DISCOVERABILITY_PARAMETER 0x07  //The given discoverability parameter does not contain a valid value according to the specification
#define ERROR_INVALID_CONNECTABILITY_PARAMETER  0x08  //The given connectability parameter does not contain a valid value according to the specification
#define ERROR_INVALID_SECURITY_MODE             0x09  //The given security mode is not a valid Bluetooth security mode
#define ERROR_LINKKEY_DOES_NOT_EXISTS           0x0a  //No link key exists for the given Bluetooth address
#define ERROR_CONNECTION_FAILED                 0x0b  //The connection setup failed due to unknown reasons
#define ERROR_TRUNCATED_ANSWER                  0x0c  //The returned number of services is too large to be handled by the LMX9820. The answer is truncated
#define ERROR_RESULT_TOO_LARGE                  0x0d  //The SDP result from the remote device is too large to be handled by the LMX9820 due to ram limitations
#define ERROR_NOT_POSSIBLE_TO_ENTER_TESTMODE    0x0e  //It is currently not possible to enter the selected test mode
#define ERROR_ILLEGAL_TESTMODE                  0x0f  //The given test mode is not a valid test mode according to the specification
#define ERROR_RESET_TO_NSC_BDADDRESS            0x10  //The LMX9820 will change the Bluetooth address to the NSC address
#define ERROR_UART_SPEED_OUT_OF_RANGE           0x11  //The selected UART speed value is not valid according to the specification
#define ERROR_INVALID_PORT                      0x12  //The given port value is larger than the maximum specified value
#define ERROR_ILLEGAL_STATE_VALUE               0x13  //The given state value is not a valid state according to the specification
#define ERROR_IDENTIFIER_OUT_OF_RANGE           0x14  //The given identifier is larger than the maximum specified value
#define ERROR_RECORD_ALREADY_IN_SELECTED_STATE  0x15  //The service record is already enabled/disabled

//ACL Error Codes
#define ERROR_INVALID_AUTHENTICATION_VALUE          0x16 //The given authentication value is not a valid value according to the specification
#define ERROR_INVALID_ENCRYPTION_VALUE              0x17 //The given encryption value is not a valid value according to the specification
#define ERROR_MAXIMUM_NO_OF_SERVICE_RECORDS_REACHED 0x18 //The maximum number of service records, which the LMX9820 is able to store, is reached
#define ERROR_WRITING_TO_NVS                        0x19 //An error occurred while writing to flash. The service record may not be stored
#define ERROR_INVALID_ROLE                          0x1a //The given role value is not a valid value according to the specification
#define ERROR_LIMIT                                 0x1b //Limits exceeded (Parameter(s) violates limits)
#define ERROR_UNEXPECTED                            0x1c //Unexpected at this moment
#define ERROR_UNABLE_TO_SEND                        0x1d //Could not send at this moment, no reason specified
#define ERROR_CURRENTLY_NO_BUFFER                   0x1e //Currently no room in buffer, try again later 
#define ERROR_NO_CONNECTION                         0x1f //Trying to use an inexistent connection
#define ERROR_SPP_INVALID_PORT                      0x20 //Port number out of range
#define ERROR_SPP_PORT_NOT_OPEN                     0x21 //Port is closed
#define ERROR_SPP_PORT_BUSY                         0x22 //Connection establishment on a PORT that has a connection.
#define ERROR_SPP_MULTIPLE_CONNECTIONS              0x23 //Transparent mode attempted while more than 1 connection active
#define ERROR_SPP_MULTIPLE_TRANSPARENT              0x24 //Trying to store a default connection when a transparent default connection is already stored, or trying to store a transparent default connection when another connection is already stored
#define ERROR_SPP_DEFAULT_CONNECTION_NOT_STORED     0x25 //Trying to connection to a default connection, which is not stored
#define ERROR_SPP_AUTOMATIC_CONNECTIONS_PROGRESSING 0x26 //Trying to start connecting to default connections when default connection establishment is already progressing.
#define ERROR_UNSPECIFIED_ERROR                     0x27 //Other error
#define ERROR_IDENTIFIER_NOT_IN_USE                 0x28 //Trying to enable a SDP record which is not stored. Wrong identifier
#define ERROR_INVALID_SUPPORTED_FAXCLASS_VALUE      0x29 //Faxclass parameter must be 0 or 1
#define ERROR_TOO_MANY_SUPPORTED_FORMATS            0x2a //The given number of supported formats excesses the specified maximum number of supported formats
#define ERROR_TOO_MANY_DATASTORES                   0x2b //The given number of data stores excess the specified maximum number of data stores
#define ERROR_ATTEMPT_FAILED                        0x2C //Attempt to change low power mode failed
#define ERROR_ILLEGAL_LINK_POLICY                   0x2D //The given link policy value is out of range
#define ERROR_PINCODE_TOO_LONG                      0x2E //The pin code length field is too large
#define ERROR_PARITY_BIT_OUT_OF_RANGE               0x2F //The given parity check is out of range
#define ERROR_STOP_BITS_OUT_OF_RANGE                0x30 //The given number of stop bits is out of range

//RFCOMM Error Codes
#define RFCS_NO_ERROR             0x00  //No error
#define RFCS_INVALID_DLC          0x01  //The DLC does not exist
#define RFCS_INVALID_PORT         0x02  //The port does not exist
#define RFCS_DLC_ESTABLISH_FAILED 0x03  //The DLC establishment failed
#define RFCS_ACCESS_REJECTED      0x04  //SECM did not authorize access to the requested service (DLC)
#define RFCS_INVALID_CONNECTION   0x05  //There does not exist a DLC/L2CAP connection to the device
#define RFCS_UNSPECIFIED_ERROR    0xFF  //Not used 

//RFComm Release Reasons 
#define RFCR_DLC_DISC_LOCAL_DEVICE  0x00  //The local device has disconnected the DLC
#define RFCR_DLC_DISC_REMOTE_DEVICE 0x01  //The remote device has disconnected the DLC
#define RFCR_DLC_DISC_ACL_FAILURE   0x02  //ACL link failure/ link supervision timeout
#define RFCR_DLC_DISC_LOWER_LAYER   0x03  //Lower layer (e.g. L2CAP) has disconnected the DLC.

