/*****************************************************************************
 *****************************************************************************
 *     _    __  __ ____  _____ ____             _          _               
 *    / \  |  \/  | __ )| ____|  _ \  __      _(_)_ __ ___| | ___  ___ ___ 
 *   / _ \ | |\/| |  _ \|  _| | |_) | \ \ /\ / / | '__/ _ \ |/ _ \/ __/ __|
 *  / ___ \| |  | | |_) | |___|  _ <   \ V  V /| | | |  __/ |  __/\__ \__ \
 * /_/   \_\_|  |_|____/|_____|_| \_\   \_/\_/ |_|_|  \___|_|\___||___/___/
 *
 *
 * Module Name:
 * ============
 *
 *			GAP_Commands.C
 *
 * Purpose:			
 * ========
 *
 *			Provide functions for GAP Commands - OK
 *
 * Author(s) & Creation Date:	
 * ==========================
 *
 *			Timo Koch, 2004/03/01
 *
 *
 * Copyright:	
 * ==========
 *
 *			AMBER wireless GmbH, Inc.
 *			Albin Koebis Strasse 18
 *			D-50858 Koeln, Germany
 *			Phone:		+49 (0) 2203-369472
 *			Fax:		+49 (0) 2203-459883
 *			Internet:	www.amber-wireless.de	
 *
 *----------------------------------------------------------------
 * All software and related documentation is provided "AS IS" and 
 * without warranty or support of any kind and AMBER wireless 
 * expressly disclaims all other warranties, express or implied, 
 * including, but not limited to, the implied warranties of 
 * merchantability and fitness for a particular purpose. Under no 
 * circumstances shall AMBER wireless be liable for any 
 * incidental, special or consequential damages that result from 
 * the use or inability to use the software or related 
 * documentation, even if AMBER wireless has been advised of 
 * the liability.
 *
 * Unless otherwise stated, software written and copyrighted by 
 * AMBER wireless is distributed as "freeware". You may use 
 * and modify this software without any charge or restriction. 
 * You may distribute to others, as long as the original author 
 * is acknowledged.
 *
 * You may not use the Program in non-AMBER devices.
 *
 *****************************************************************************
 *****************************************************************************/

#include "GAP_Commands.h"

/*****************************************************************************
 * Macros & Typedefs							     *
 *****************************************************************************/

/********************************
 * Constant macros		*
 ********************************/

/********************************
 * Function macros		*
 ********************************/

/********************************
 * Typedefs			*
 ********************************/

/*****************************************************************************
 * Static / Non-static global declarations				     *
 *****************************************************************************/

/********************************
 * Non-static globals 		*
 ********************************/

/********************************
 * Static globals		*
 ********************************/


/*****************************************************************************
 * Function declarations	                			     *
 *****************************************************************************/
uint8_t GAP_Inquiry_REQ                (GAP_INQUIRY_REQ_t* GAP_Request);
uint8_t GAP_Remote_Device_Name_REQ     (GAP_REMOTE_DEVICE_NAME_REQ_t* GAP_Request);
uint8_t GAP_Set_Link_Policy_REQ        (GAP_SET_LINK_POLICY_REQ_t* GAP_Request);
uint8_t GAP_Get_Link_Policy_REQ        (GAP_GET_LINK_POLICY_REQ_t* GAP_Request);
uint8_t GAP_Enter_Sniff_Mode_REQ       (GAP_ENTER_SNIFF_MODE_REQ_t* GAP_Request);
uint8_t GAP_Exit_Sniff_Mode_REQ        (GAP_EXIT_SNIFF_MODE_REQ_t* GAP_Request);
uint8_t GAP_Enter_Park_Mode_REQ        (GAP_ENTER_PARK_MODE_REQ_t* GAP_Request);
uint8_t GAP_Exit_Park_Mode_REQ         (GAP_EXIT_PARK_MODE_REQ_t* GAP_Request);
uint8_t GAP_Enter_Hold_Mode_REQ        (GAP_ENTER_HOLD_MODE_REQ_t* GAP_Request);
uint8_t GAP_Read_Local_Name_REQ        (void);
uint8_t GAP_Write_Local_Name_REQ       (GAP_WRITE_LOCAL_NAME_REQ_t* GAP_Request);
uint8_t GAP_Read_Local_BDA_REQ         (void);
uint8_t GAP_Set_Scanmode_REQ           (GAP_SET_SCANMODE_REQ_t* GAP_Request);
uint8_t GAP_Get_Security_Mode_REQ      (void);
uint8_t GAP_Set_Security_Mode_REQ      (GAP_SET_SECURITY_MODE_REQ_t* GAP_Request);
uint8_t GAP_Remove_Pairing_REQ         (GAP_REMOVE_PAIRING_REQ_t* GAP_Request);
uint8_t GAP_List_Paired_Devices_REQ    (void);
uint8_t GAP_Get_Fixed_Pin_REQ          (void);
uint8_t GAP_Set_Fixed_Pin_REQ          (GAP_SET_FIXED_PIN_REQ_t* GAP_Request);

/*****************************************************************************
 * Function definitions							     * 
 *****************************************************************************/

uint8_t GAP_Inquiry_REQ (GAP_INQUIRY_REQ_t* GAP_Request)
{
  
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(GAP_INQUIRY);                               // Op_code
  UART1_TX_WriteByte(3);                                         // Data_length_LSB 
  UART1_TX_WriteByte(0);                                         // Data_length_MSB
  UART1_TX_WriteByte((uint8_t)(REQ + GAP_INQUIRY + 3));             // Checksum
  UART1_TX_WriteByte(GAP_Request->Length);                       // Data
  UART1_TX_WriteByte(GAP_Request->NumResponses);                 // Data
  UART1_TX_WriteByte(GAP_Request->Mode);                         // Data
  UART1_TX_WriteByte(ETX);                                       // End_delimiter
  
  return (0);                                                    // OK
} 


uint8_t GAP_Remote_Device_Name_REQ (GAP_REMOTE_DEVICE_NAME_REQ_t* GAP_Request)
{
  
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(GAP_REMOTE_DEVICE_NAME);                    // Op_code
  UART1_TX_WriteByte(6);                                         // Data_length_LSB 
  UART1_TX_WriteByte(0);                                         // Data_length_MSB
  UART1_TX_WriteByte((uint8_t)(REQ + GAP_REMOTE_DEVICE_NAME + 6));  // Checksum
  UART1_TX_WriteByte(GAP_Request->BdAddr.Upper_Part_MSB);        // Data
  UART1_TX_WriteByte(GAP_Request->BdAddr.Upper_Part_LSB);        // Data
  UART1_TX_WriteByte(GAP_Request->BdAddr.Middle_Part_MSB);       // Data
  UART1_TX_WriteByte(GAP_Request->BdAddr.Middle_Part_LSB);       // Data
  UART1_TX_WriteByte(GAP_Request->BdAddr.Lower_Part_MSB);        // Data
  UART1_TX_WriteByte(GAP_Request->BdAddr.Lower_Part_LSB);        // Data
  UART1_TX_WriteByte(ETX);                                       // End_delimiter
  
  return (0);                                                               // OK
} 



uint8_t GAP_Set_Link_Policy_REQ (GAP_SET_LINK_POLICY_REQ_t* GAP_Request)
{
        
  
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(GAP_SET_LINK_POLICY);                       // Op_code
  UART1_TX_WriteByte(8);                                         // Data_length_LSB
  UART1_TX_WriteByte(0);                                         // Data_length_MSB 
  UART1_TX_WriteByte((uint8_t)(REQ + GAP_SET_LINK_POLICY + 8));     // Checksum
  UART1_TX_WriteByte(GAP_Request->BdAddr.Upper_Part_MSB);        // Data
  UART1_TX_WriteByte(GAP_Request->BdAddr.Upper_Part_LSB);        // Data
  UART1_TX_WriteByte(GAP_Request->BdAddr.Middle_Part_MSB);       // Data
  UART1_TX_WriteByte(GAP_Request->BdAddr.Middle_Part_LSB);       // Data
  UART1_TX_WriteByte(GAP_Request->BdAddr.Lower_Part_MSB);        // Data
  UART1_TX_WriteByte(GAP_Request->BdAddr.Lower_Part_LSB);        // Data
  UART1_TX_WriteByte((uint8_t)(GAP_Request->LinkPolicy>>8));        // Data
  UART1_TX_WriteByte((uint8_t)(GAP_Request->LinkPolicy));           // Data
  
  UART1_TX_WriteByte(ETX);                                       // End_delimiter
  
  return (0);                                                               // OK
} 



uint8_t GAP_Get_Link_Policy_REQ (GAP_GET_LINK_POLICY_REQ_t* GAP_Request)
{
  
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(GAP_GET_LINK_POLICY);                       // Op_code
  UART1_TX_WriteByte(6);                                         // Data_length_LSB
  UART1_TX_WriteByte(0);                                         // Data_length_MSB 
  UART1_TX_WriteByte((uint8_t)(REQ + GAP_GET_LINK_POLICY + 6));     // Checksum
  UART1_TX_WriteByte(GAP_Request->BdAddr.Upper_Part_MSB);        // Data
  UART1_TX_WriteByte(GAP_Request->BdAddr.Upper_Part_LSB);        // Data
  UART1_TX_WriteByte(GAP_Request->BdAddr.Middle_Part_MSB);       // Data
  UART1_TX_WriteByte(GAP_Request->BdAddr.Middle_Part_LSB);       // Data
  UART1_TX_WriteByte(GAP_Request->BdAddr.Lower_Part_MSB);        // Data
  UART1_TX_WriteByte(GAP_Request->BdAddr.Lower_Part_LSB);        // Data
  
  UART1_TX_WriteByte(ETX);                                       // End_delimiter
  
  return (0);                                                               // OK
} 


uint8_t GAP_Enter_Sniff_Mode_REQ (GAP_ENTER_SNIFF_MODE_REQ_t* GAP_Request)
{
    
  
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(GAP_ENTER_SNIFF_MODE);                      // Op_code
  UART1_TX_WriteByte(14);                                        // Data_length_LSB
  UART1_TX_WriteByte(0);                                         // Data_length_MSB 
  UART1_TX_WriteByte((uint8_t)(REQ + GAP_ENTER_SNIFF_MODE + 14));   // Checksum
  UART1_TX_WriteByte(GAP_Request->BdAddr.Upper_Part_MSB);        // Data
  UART1_TX_WriteByte(GAP_Request->BdAddr.Upper_Part_LSB);        // Data
  UART1_TX_WriteByte(GAP_Request->BdAddr.Middle_Part_MSB);       // Data
  UART1_TX_WriteByte(GAP_Request->BdAddr.Middle_Part_LSB);       // Data
  UART1_TX_WriteByte(GAP_Request->BdAddr.Lower_Part_MSB);        // Data
  UART1_TX_WriteByte(GAP_Request->BdAddr.Lower_Part_LSB);        // Data
  UART1_TX_WriteByte((uint8_t)(GAP_Request->SniffMaxInterval>>8));  // Data
  UART1_TX_WriteByte((uint8_t)(GAP_Request->SniffMaxInterval));     // Data
  UART1_TX_WriteByte((uint8_t)(GAP_Request->SniffMinInterval>>8));  // Data
  UART1_TX_WriteByte((uint8_t)(GAP_Request->SniffMinInterval));     // Data
  UART1_TX_WriteByte((uint8_t)(GAP_Request->SniffAttempt>>8));      // Data
  UART1_TX_WriteByte((uint8_t)(GAP_Request->SniffAttempt));         // Data
  UART1_TX_WriteByte((uint8_t)(GAP_Request->SniffTimeout>>8));      // Data
  UART1_TX_WriteByte((uint8_t)(GAP_Request->SniffTimeout));         // Data
  UART1_TX_WriteByte(ETX);                                       // End_delimiter
  
  return (0);                                                               // OK
} 

uint8_t GAP_Exit_Sniff_Mode_REQ (GAP_EXIT_SNIFF_MODE_REQ_t* GAP_Request)
{
  
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(GAP_EXIT_SNIFF_MODE);                       // Op_code
  UART1_TX_WriteByte(6);                                         // Data_length_LSB
  UART1_TX_WriteByte(0);                                         // Data_length_MSB 
  UART1_TX_WriteByte((uint8_t)(REQ + GAP_EXIT_SNIFF_MODE + 6));     // Checksum
  UART1_TX_WriteByte(GAP_Request->BdAddr.Upper_Part_MSB);        // Data
  UART1_TX_WriteByte(GAP_Request->BdAddr.Upper_Part_LSB);        // Data
  UART1_TX_WriteByte(GAP_Request->BdAddr.Middle_Part_MSB);       // Data
  UART1_TX_WriteByte(GAP_Request->BdAddr.Middle_Part_LSB);       // Data
  UART1_TX_WriteByte(GAP_Request->BdAddr.Lower_Part_MSB);        // Data
  UART1_TX_WriteByte(GAP_Request->BdAddr.Lower_Part_LSB);        // Data
  UART1_TX_WriteByte(ETX);                                       // End_delimiter
  
  return (0);                                                               // OK
} 

 
uint8_t GAP_Enter_Park_Mode_REQ (GAP_ENTER_PARK_MODE_REQ_t* GAP_Request)
{
  
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(GAP_ENTER_PARK_MODE);                       // Op_code
  UART1_TX_WriteByte(10);                                        // Data_length_LSB
  UART1_TX_WriteByte(0);                                         // Data_length_MSB 
  UART1_TX_WriteByte((uint8_t)(REQ + GAP_ENTER_PARK_MODE + 10));    // Checksum
  UART1_TX_WriteByte(GAP_Request->BdAddr.Upper_Part_MSB);        // Data
  UART1_TX_WriteByte(GAP_Request->BdAddr.Upper_Part_LSB);        // Data
  UART1_TX_WriteByte(GAP_Request->BdAddr.Middle_Part_MSB);       // Data
  UART1_TX_WriteByte(GAP_Request->BdAddr.Middle_Part_LSB);       // Data
  UART1_TX_WriteByte(GAP_Request->BdAddr.Lower_Part_MSB);        // Data
  UART1_TX_WriteByte(GAP_Request->BdAddr.Lower_Part_LSB);        // Data
  UART1_TX_WriteByte((uint8_t)(GAP_Request->BeaconMaxInterval>>8)); // Data
  UART1_TX_WriteByte((uint8_t)(GAP_Request->BeaconMaxInterval));    // Data
  UART1_TX_WriteByte((uint8_t)(GAP_Request->BeaconMinInterval>>8)); // Data
  UART1_TX_WriteByte((uint8_t)(GAP_Request->BeaconMinInterval));    // Data
  UART1_TX_WriteByte(ETX);                                       // End_delimiter
  
  return (0);                                                               // OK
} 
  

uint8_t GAP_Exit_Park_Mode_REQ (GAP_EXIT_PARK_MODE_REQ_t* GAP_Request)
{
          
  
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(GAP_EXIT_PARK_MODE);                        // Op_code
  UART1_TX_WriteByte(6);                                         // Data_length_LSB
  UART1_TX_WriteByte(0);                                         // Data_length_MSB 
  UART1_TX_WriteByte((uint8_t)(REQ + GAP_EXIT_PARK_MODE + 6));      // Checksum
  UART1_TX_WriteByte(GAP_Request->BdAddr.Upper_Part_MSB);        // Data
  UART1_TX_WriteByte(GAP_Request->BdAddr.Upper_Part_LSB);        // Data
  UART1_TX_WriteByte(GAP_Request->BdAddr.Middle_Part_MSB);       // Data
  UART1_TX_WriteByte(GAP_Request->BdAddr.Middle_Part_LSB);       // Data
  UART1_TX_WriteByte(GAP_Request->BdAddr.Lower_Part_MSB);        // Data
  UART1_TX_WriteByte(GAP_Request->BdAddr.Lower_Part_LSB);        // Data
  UART1_TX_WriteByte(ETX);                                       // End_delimiter
  
  return (0);                                                               // OK
} 



uint8_t GAP_Enter_Hold_Mode_REQ (GAP_ENTER_HOLD_MODE_REQ_t* GAP_Request)
{
          
  
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(GAP_ENTER_HOLD_MODE);                       // Op_code
  UART1_TX_WriteByte(10);                                        // Data_length_LSB
  UART1_TX_WriteByte(0);                                         // Data_length_MSB 
  UART1_TX_WriteByte((uint8_t)(REQ + GAP_ENTER_HOLD_MODE + 10));    // Checksum
  UART1_TX_WriteByte(GAP_Request->BdAddr.Upper_Part_MSB);        // Data
  UART1_TX_WriteByte(GAP_Request->BdAddr.Upper_Part_LSB);        // Data
  UART1_TX_WriteByte(GAP_Request->BdAddr.Middle_Part_MSB);       // Data
  UART1_TX_WriteByte(GAP_Request->BdAddr.Middle_Part_LSB);       // Data
  UART1_TX_WriteByte(GAP_Request->BdAddr.Lower_Part_MSB);        // Data
  UART1_TX_WriteByte(GAP_Request->BdAddr.Lower_Part_LSB);        // Data
  UART1_TX_WriteByte((uint8_t)(GAP_Request->HoldMaxInterval>>8)); // Data
  UART1_TX_WriteByte((uint8_t)(GAP_Request->HoldMaxInterval));    // Data
  UART1_TX_WriteByte((uint8_t)(GAP_Request->HoldMinInterval>>8)); // Data
  UART1_TX_WriteByte((uint8_t)(GAP_Request->HoldMinInterval));    // Data
  UART1_TX_WriteByte(ETX);                                       // End_delimiter
  
  return (0);                                                               // OK
} 



uint8_t GAP_Read_Local_Name_REQ (void)
{
      
  
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(GAP_READ_LOCAL_NAME);                       // Op_code
  UART1_TX_WriteByte(0);                                         // Data_length_MSB 
  UART1_TX_WriteByte(0);                                         // Data_length_LSB
  UART1_TX_WriteByte((uint8_t)(REQ + GAP_READ_LOCAL_NAME + 0));     // Checksum
  
  UART1_TX_WriteByte(ETX);                                       // End_delimiter
  
  return (0);                                                               // OK
}


uint8_t GAP_Write_Local_Name_REQ (GAP_WRITE_LOCAL_NAME_REQ_t* GAP_Request)
{
uint8_t i;
  
  i = GAP_Request->NameLength + FRAME_OVERHEAD + 1;

             
  
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(GAP_WRITE_LOCAL_NAME);                      // Op_code
  i = GAP_Request->NameLength + 1;                                          // Calc Length
  UART1_TX_WriteByte((uint8_t)i);                                   // Data_length_LSB
  UART1_TX_WriteByte((uint8_t)(i>>8));                              // Data_length_MSB 
  UART1_TX_WriteByte((uint8_t)(REQ + GAP_WRITE_LOCAL_NAME + i));    // Checksum
  UART1_TX_WriteByte(GAP_Request->NameLength);                   // Data
  
  for(i=0;i < GAP_Request->NameLength;i++)
    UART1_TX_WriteByte(GAP_Request->DeviceName[i]); // Data
  
  UART1_TX_WriteByte(ETX);                                       // End_delimiter
  
  return (0);                                                               // OK
}


uint8_t GAP_Read_Local_BDA_REQ (void)
{
          
  
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(GAP_READ_LOCAL_BDA);                        // Op_code
  UART1_TX_WriteByte(0);                                         // Data_length_MSB 
  UART1_TX_WriteByte(0);                                         // Data_length_LSB
  UART1_TX_WriteByte((uint8_t)(REQ + GAP_READ_LOCAL_BDA + 0));      // Checksum
  
  UART1_TX_WriteByte(ETX);                                       // End_delimiter
  
  return (0);                                                               // OK
}

uint8_t GAP_Set_Scanmode_REQ (GAP_SET_SCANMODE_REQ_t* GAP_Request)
{
    
  
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(GAP_SET_SCANMODE);                          // Op_code
  UART1_TX_WriteByte(2);                                         // Data_length_LSB
  UART1_TX_WriteByte(0);                                         // Data_length_MSB 
  UART1_TX_WriteByte((uint8_t)(REQ + GAP_SET_SCANMODE + 2));        // Checksum
  UART1_TX_WriteByte(GAP_Request->Connectability);               // Data
  UART1_TX_WriteByte(GAP_Request->Discoverability);              // Data
  UART1_TX_WriteByte(ETX);                                       // End_delimiter
  
  return (0);                                                               // OK
}

uint8_t GAP_Get_Security_Mode_REQ (void)
{
  
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(GAP_GET_SECURITY_MODE);                     // Op_code
  UART1_TX_WriteByte(0);                                         // Data_length_MSB 
  UART1_TX_WriteByte(0);                                         // Data_length_LSB
  UART1_TX_WriteByte((uint8_t)(REQ + GAP_GET_SECURITY_MODE + 0));   // Checksum
  
  UART1_TX_WriteByte(ETX);                                       // End_delimiter
  
  return (0);                                                               // OK
}


uint8_t GAP_Set_Security_Mode_REQ (GAP_SET_SECURITY_MODE_REQ_t* GAP_Request)
{
  
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(GAP_SET_SECURITY_MODE);                     // Op_code
  UART1_TX_WriteByte(1);                                         // Data_length_LSB
  UART1_TX_WriteByte(0);                                         // Data_length_MSB 
  UART1_TX_WriteByte((uint8_t)(REQ + GAP_SET_SECURITY_MODE + 1));   // Checksum
  UART1_TX_WriteByte(GAP_Request->Mode);                         // Data
  UART1_TX_WriteByte(ETX);                                       // End_delimiter
  
  return (0);                                                               // OK
}


uint8_t GAP_Remove_Pairing_REQ (GAP_REMOVE_PAIRING_REQ_t* GAP_Request)
{
  
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(GAP_REMOVE_PAIRING);                        // Op_code
  UART1_TX_WriteByte(6);                                         // Data_length_LSB 
  UART1_TX_WriteByte(0);                                         // Data_length_MSB
  UART1_TX_WriteByte((uint8_t)(REQ + GAP_REMOVE_PAIRING + 6));      // Checksum
  UART1_TX_WriteByte(GAP_Request->BdAddr.Upper_Part_MSB);        // Data
  UART1_TX_WriteByte(GAP_Request->BdAddr.Upper_Part_LSB);        // Data
  UART1_TX_WriteByte(GAP_Request->BdAddr.Middle_Part_MSB);       // Data
  UART1_TX_WriteByte(GAP_Request->BdAddr.Middle_Part_LSB);       // Data
  UART1_TX_WriteByte(GAP_Request->BdAddr.Lower_Part_MSB);        // Data
  UART1_TX_WriteByte(GAP_Request->BdAddr.Lower_Part_LSB);        // Data
  UART1_TX_WriteByte(ETX);                                       // End_delimiter
  
  return (0);                                                               // OK
}


uint8_t GAP_List_Paired_Devices_REQ (void)
{
 
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(GAP_LIST_PAIRED_DEVICES);                   // Op_code
  UART1_TX_WriteByte(0);                                         // Data_length_MSB 
  UART1_TX_WriteByte(0);                                         // Data_length_LSB
  UART1_TX_WriteByte((uint8_t)(REQ + GAP_LIST_PAIRED_DEVICES + 0)); // Checksum
  
  UART1_TX_WriteByte(ETX);                                       // End_delimiter
  
  return (0);                                                               // OK
}



uint8_t GAP_Get_Fixed_Pin_REQ (void)
{
  
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(GAP_GET_FIXED_PIN);                         // Op_code
  UART1_TX_WriteByte(0);                                         // Data_length_MSB 
  UART1_TX_WriteByte(0);                                         // Data_length_LSB
  UART1_TX_WriteByte((uint8_t)(REQ + GAP_GET_FIXED_PIN + 0));       // Checksum
  
  UART1_TX_WriteByte(ETX);                                       // End_delimiter
  
  return (0);                                                               // OK
}



uint8_t GAP_Set_Fixed_Pin_REQ (GAP_SET_FIXED_PIN_REQ_t* GAP_Request)
{
uint8_t i;
  i = GAP_Request->Pinlength + FRAME_OVERHEAD + 1;

  
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(GAP_SET_FIXED_PIN);                         // Op_code
  i = GAP_Request->Pinlength + 1;                                           // Calc Length
  UART1_TX_WriteByte((uint8_t)i);                              // Data_length_MSB 
  UART1_TX_WriteByte((uint8_t)(i>>8));                                   // Data_length_LSB
  UART1_TX_WriteByte((uint8_t)(REQ + GAP_SET_FIXED_PIN + i));       // Checksum
  UART1_TX_WriteByte(GAP_Request->Pinlength);                    // Data
  
  for(i=0;i < GAP_Request->Pinlength;i++)
    UART1_TX_WriteByte(GAP_Request->Pincode[i]);    // Data
  
  UART1_TX_WriteByte(ETX);                                       // End_delimiter
  
  return (0);                                                               // OK
}




