/*****************************************************************************
 *****************************************************************************
 *     _    __  __ ____  _____ ____             _          _               
 *    / \  |  \/  | __ )| ____|  _ \  __      _(_)_ __ ___| | ___  ___ ___ 
 *   / _ \ | |\/| |  _ \|  _| | |_) | \ \ /\ / / | '__/ _ \ |/ _ \/ __/ __|
 *  / ___ \| |  | | |_) | |___|  _ <   \ V  V /| | | |  __/ |  __/\__ \__ \
 * /_/   \_\_|  |_|____/|_____|_| \_\   \_/\_/ |_|_|  \___|_|\___||___/___/
 *
 *																			 
 * Module Name:																  
 * ============
 *
 *			GAP_Commands.h
 *
 * Purpose:			
 * ========
 *
 *			header file for GAP_Commands.c
 *
 * Author(s) & Creation Date:	
 * ==========================
 *
 *			Timo Koch, 2004/03/01
 *
 *
 * Copyright:	
 * ==========
 *
 *			AMBER wireless GmbH, Inc.
 *			Albin Koebis Strasse 18
 *			D-50858 Koeln, Germany
 *			Phone:		+49 (0) 2203-369472
 *			Fax:		+49 (0) 2203-459883
 *			Internet:	www.amber-wireless.de	
 *
 *----------------------------------------------------------------
 * All software and related documentation is provided "AS IS" and 
 * without warranty or support of any kind and AMBER wireless 
 * expressly disclaims all other warranties, express or implied, 
 * including, but not limited to, the implied warranties of 
 * merchantability and fitness for a particular purpose. Under no 
 * circumstances shall AMBER wireless be liable for any 
 * incidental, special or consequential damages that result from 
 * the use or inability to use the software or related 
 * documentation, even if AMBER wireless has been advised of 
 * the liability.
 *
 * Unless otherwise stated, software written and copyrighted by 
 * AMBER wireless is distributed as "freeware". You may use 
 * and modify this software without any charge or restriction. 
 * You may distribute to others, as long as the original author 
 * is acknowledged.
 *
 * You may not use the Program in non-AMBER devices.
 *
 *****************************************************************************
 *****************************************************************************/




#ifndef _GAP_COMMANDS_H_
#define _GAP_COMMANDS_H_

#include "main.h"         // provides UART1_TX_WriteByte 
#include "LMX_Std.h"      
#include "MY_std.h"

/*****************************************************************************
 * Macros & Typedefs							     *
 *****************************************************************************/


/********************************
 * Constant macros		*
 ********************************/

#define NAME_LENGTH_REMOTE_DEVICE     30        // Max 30 bytes including NULL termination
#define DEVICE_NAME_LENGHT            30        // Max 30 bytes including NULL termination

// Opcodes
#define GAP_INQUIRY                     0x00
#define GAP_DEVICE_FOUND                0x01
#define GAP_REMOTE_DEVICE_NAME          0x02
#define GAP_READ_LOCAL_NAME             0x03
#define GAP_WRITE_LOCAL_NAME            0x04
#define GAP_READ_LOCAL_BDA              0x05
#define GAP_SET_SCANMODE                0x06
#define GAP_GET_FIXED_PIN               0x16
#define GAP_SET_FIXED_PIN               0x17
#define GAP_GET_SECURITY_MODE           0x18
#define GAP_SET_SECURITY_MODE           0x19
#define GAP_REMOVE_PAIRING              0x1B
#define GAP_LIST_PAIRED_DEVICES         0x1C
#define GAP_ENTER_SNIFF_MODE            0x21
#define GAP_EXIT_SNIFF_MODE             0x37
#define GAP_ENTER_PARK_MODE             0x38
#define GAP_EXIT_PARK_MODE              0x39
#define GAP_ENTER_HOLD_MODE             0x3A
#define GAP_SET_LINK_POLICY             0x3B
#define GAP_GET_LINK_POLICY             0x3C
#define GAP_POWER_SAVE_MODE_CHANGED     0x3D
#define GAP_ACL_ESTABLISHED             0x50
#define GAP_ACL_TERMINATED              0x51


// Msg identification ( OP-Code + Packet type)
// easier and safer handle of incoming messages
/*
Packet type identification
#define REQ     0x52
#define CFM     0x43
#define IND     0x69
#define RES     0x72
*/
#define GAP_INQUIRY_REQ                 0x0052
#define GAP_INQUIRY_CFM                 0x0043
#define GAP_DEVICE_FOUND_IND            0x0169

#define GAP_REMOTE_DEVICE_NAME_REQ      0x0252
#define GAP_REMOTE_DEVICE_NAME_CFM      0x0243

#define GAP_READ_LOCAL_NAME_REQ         0x0352
#define GAP_READ_LOCAL_NAME_CFM         0x0343

#define GAP_WRITE_LOCAL_NAME_REQ        0x0452
#define GAP_WRITE_LOCAL_NAME_CFM        0x0443

#define GAP_READ_LOCAL_BDA_REQ          0x0552
#define GAP_READ_LOCAL_BDA_CFM          0x0543

#define GAP_SET_SCANMODE_REQ            0x0652
#define GAP_SET_SCANMODE_CFM            0x0643

#define GAP_GET_FIXED_PIN_REQ           0x1652
#define GAP_GET_FIXED_PIN_CFM           0x1643

#define GAP_SET_FIXED_PIN_REQ           0x1752
#define GAP_SET_FIXED_PIN_CFM           0x1743

#define GAP_GET_SECURITY_MODE_REQ       0x1852
#define GAP_GET_SECURITY_MODE_CFM       0x1843

#define GAP_SET_SECURITY_MODE_REQ       0x1952
#define GAP_SET_SECURITY_MODE_CFM       0x1943

#define GAP_REMOVE_PAIRING_REQ          0x1B52
#define GAP_REMOVE_PAIRING_CFM          0x1B43

#define GAP_LIST_PAIRED_DEVICES_REQ     0x1C52
#define GAP_LIST_PAIRED_DEVICES_CFM     0x1C43

#define GAP_ENTER_SNIFF_MODE_REQ        0x2152
#define GAP_ENTER_SNIFF_MODE_CFM        0x2143

#define GAP_EXIT_SNIFF_MODE_REQ         0x3752
#define GAP_EXIT_SNIFF_MODE_CFM         0x3743

#define GAP_ENTER_PARK_MODE_REQ         0x3852
#define GAP_ENTER_PARK_MODE_CFM         0x3843

#define GAP_EXIT_PARK_MODE_REQ          0x3952
#define GAP_EXIT_PARK_MODE_CFM          0x3943

#define GAP_ENTER_HOLD_MODE_REQ         0x3A52
#define GAP_ENTER_HOLD_MODE_CFM         0x3A43

#define GAP_SET_LINK_POLICY_REQ         0x3B52
#define GAP_SET_LINK_POLICY_CFM         0x3B43

#define GAP_GET_LINK_POLICY_REQ         0x3C52
#define GAP_GET_LINK_POLICY_CFM         0x3C43

#define GAP_POWER_SAVE_MODE_CHANGED_REQ 0x3D52
#define GAP_POWER_SAVE_MODE_CHANGED_CFM 0x3D43

#define GAP_ACL_ESTABLISHED_REQ         0x5052
#define GAP_ACL_ESTABLISHED_CFM         0x5043

#define GAP_ACL_TERMINATED_REQ          0x5152
#define GAP_ACL_TERMINATED_CFM          0x5143

/********************************
 * Function macros		*
 ********************************/

/********************************
 * Typedefs			*
 ********************************/

/********************************
 * GAP Inquiry 			*
 ********************************/
typedef struct 
{
  uint8_t Length;
  uint8_t NumResponses;
  uint8_t Mode;
}GAP_INQUIRY_REQ_t;

typedef struct 
{
  uint8_t Status;
}GAP_INQUIRY_CFM_t;

typedef struct 
{
  BD_ADDR_t     BdAddr;
  uint32_t DeviceClass;
}GAP_DEVICE_FOUND_IND_t;


/********************************
 * GAP Remote Device Name       *
 ********************************/
typedef struct
{
  BD_ADDR_t BdAddr;
}GAP_REMOTE_DEVICE_NAME_REQ_t;

typedef struct
{
  uint8_t Status;
  BD_ADDR_t     BdAddr;
  uint8_t NameLength;
  uint8_t DeviceName[NAME_LENGTH_REMOTE_DEVICE];
}GAP_REMOTE_DEVICE_NAME_CFM_t;


/********************************
 * GAP Set Link Policy          *
 ********************************/
typedef struct
{
  BD_ADDR_t BdAddr;
  uint16_t LinkPolicy;
}GAP_SET_LINK_POLICY_REQ_t;

typedef struct
{
  uint8_t Status;
}GAP_SET_LINK_POLICY_CFM_t;


/********************************
 * GAP Get Link Policy          *
 ********************************/
typedef struct
{
  BD_ADDR_t BdAddr;
}GAP_GET_LINK_POLICY_REQ_t;

typedef struct
{ 
  uint8_t Status;
  uint16_t  LinkPolicy;
}GAP_GET_LINK_POLICY_CFM_t;


/********************************
 * GAP Enter Sniff Mode         *
 ********************************/
typedef struct
{
  BD_ADDR_t    BdAddr;
  uint16_t SniffMaxInterval;
  uint16_t SniffMinInterval;
  uint16_t SniffAttempt;
  uint16_t SniffTimeout;  
}GAP_ENTER_SNIFF_MODE_REQ_t;

typedef struct
{ 
  uint8_t Status;
}GAP_ENTER_SNIFF_MODE_CFM_t;


/********************************
 * GAP Exit Sniff Mode          *
 ********************************/
typedef struct
{
  BD_ADDR_t BdAddr;
}GAP_EXIT_SNIFF_MODE_REQ_t;

typedef struct
{ 
  uint8_t Status;
}GAP_EXIT_SNIFF_MODE_CFM_t;


/********************************
 * GAP Enter Park Mode          *
 ********************************/
typedef struct
{
  BD_ADDR_t    BdAddr;
  uint16_t BeaconMaxInterval;
  uint16_t BeaconMinInterval;
}GAP_ENTER_PARK_MODE_REQ_t;

typedef struct
{ 
  uint8_t Status;
}GAP_ENTER_PARK_MODE_CFM_t;


/********************************
 * GAP Exit Park Mode           *
 ********************************/
typedef struct
{
  BD_ADDR_t BdAddr;
}GAP_EXIT_PARK_MODE_REQ_t;

typedef struct
{ 
  uint8_t Status;
}GAP_EXIT_PARK_MODE_CFM_t;


/********************************
 * GAP Enter Hold Mode          *
 ********************************/
typedef struct
{
  BD_ADDR_t    BdAddr;
  uint16_t HoldMaxInterval;
  uint16_t HoldMinInterval;
}GAP_ENTER_HOLD_MODE_REQ_t;

typedef struct
{ 
  uint8_t Status;
}GAP_ENTER_HOLD_MODE_CFM_t;


/********************************
 * GAP Power Save Mode changed  *
 ********************************/
typedef struct
{
  uint8_t Status;  
  BD_ADDR_t     BdAddr;
}GAP_POWER_SAVE_MODE_CHANGED_IND_t;


/********************************
 * GAP Write Local Name         *
 ********************************/
typedef struct
{
  uint8_t NameLength;
  uint8_t DeviceName[DEVICE_NAME_LENGHT];
}GAP_WRITE_LOCAL_NAME_REQ_t;

typedef struct
{ 
  uint8_t Status;
}GAP_WRITE_LOCAL_NAME_CFM_t;


/********************************
 * GAP Read Local Name          *
 ********************************/
typedef struct
{ 
  uint8_t Status;
  uint8_t NameLength;
  uint8_t DeviceName[DEVICE_NAME_LENGHT];
}GAP_READ_LOCAL_NAME_CFM_t;


/********************************
 * GAP Set Scan Mode            *
 ********************************/
typedef struct 
{
  uint8_t Connectability;
  uint8_t Discoverability;
}GAP_SET_SCANMODE_REQ_t;

typedef struct
{ 
  uint8_t Status;
}GAP_SET_SCANMODE_CFM_t;


/********************************
 * GAP Set Security Mode        *
 ********************************/
typedef struct
{
  uint8_t Mode;
}GAP_SET_SECURITY_MODE_REQ_t;

typedef struct
{ 
  uint8_t Status;
}GAP_SET_SECURITY_MODE_CFM_t;


/********************************
 * GAP Get Security Mode        *
 ********************************/
typedef struct
{
  uint8_t Status;
  uint8_t Mode;
}GAP_GET_SECURITY_MODE_REQ_t;


/********************************
 * GAP Remove Pairing           *
 ********************************/
typedef struct
{
  BD_ADDR_t BdAddr;
}GAP_REMOVE_PAIRING_REQ_t;

typedef struct
{ 
  uint8_t Status;
}GAP_REMOVE_PAIRING_CFM_t;


/********************************
 * GAP Set fixed Pin            *
 ********************************/
typedef struct
{
  uint8_t Pinlength;
  uint8_t Pincode[PIN_LENGHT];
}GAP_SET_FIXED_PIN_REQ_t;

typedef struct
{ 
  uint8_t Status;
}GAP_SET_FIXED_PIN_CFM_t;


/********************************
 * GAP Get fixed Pin            *
 ********************************/
typedef struct
{
  uint8_t Status;
  uint8_t Pinlength;
  uint8_t Pincode[PIN_LENGHT];
}GAP_GET_FIXED_PIN_CFM_t;


/********************************
 * GAP Read local BDA           *
 ********************************/
typedef struct 
{
  uint8_t Status;
  BD_ADDR_t     BdAddr;
}GAP_READ_LOCAL_BDA_CFM_t;

/*****************************************************************************
 * Exported Variables & Functions					     *
 *****************************************************************************/

/********************************
 * Exported Variables		*
 ********************************/

/********************************
 * Exported Functions		*
 ********************************/

extern uint8_t GAP_Inquiry_REQ              (GAP_INQUIRY_REQ_t*);
extern uint8_t GAP_Remote_Device_Name_REQ   (GAP_REMOTE_DEVICE_NAME_REQ_t*);
extern uint8_t GAP_Set_Link_Policy_REQ      (GAP_SET_LINK_POLICY_REQ_t*);
extern uint8_t GAP_Get_Link_Policy_REQ      (GAP_GET_LINK_POLICY_REQ_t*);
extern uint8_t GAP_Enter_Sniff_Mode_REQ     (GAP_ENTER_SNIFF_MODE_REQ_t*);
extern uint8_t GAP_Exit_Sniff_Mode_REQ      (GAP_EXIT_SNIFF_MODE_REQ_t*);
extern uint8_t GAP_Enter_Park_Mode_REQ      (GAP_ENTER_PARK_MODE_REQ_t*);
extern uint8_t GAP_Exit_Park_Mode_REQ       (GAP_EXIT_PARK_MODE_REQ_t*);
extern uint8_t GAP_Enter_Hold_Mode_REQ      (GAP_ENTER_HOLD_MODE_REQ_t*);
extern uint8_t GAP_Read_Local_Name_REQ      (void);
extern uint8_t GAP_Write_Local_Name_REQ     (GAP_WRITE_LOCAL_NAME_REQ_t*);
extern uint8_t GAP_Read_Local_BDA_REQ       (void);
extern uint8_t GAP_Set_Scanmode_REQ         (GAP_SET_SCANMODE_REQ_t*);
extern uint8_t GAP_Get_Security_Mode_REQ    (void);
extern uint8_t GAP_Set_Security_Mode_REQ    (GAP_SET_SECURITY_MODE_REQ_t*);
extern uint8_t GAP_Remove_Pairing_REQ       (GAP_REMOVE_PAIRING_REQ_t*);
extern uint8_t GAP_List_Paired_Devices_REQ  (void);
extern uint8_t GAP_Get_Fixed_Pin_REQ        (void);
extern uint8_t GAP_Set_Fixed_Pin_REQ        (GAP_SET_FIXED_PIN_REQ_t*);

#endif /* #ifdef _GAP_COMMANDS_H_ */

