/*****************************************************************************
 *****************************************************************************
 *     _    __  __ ____  _____ ____             _          _               
 *    / \  |  \/  | __ )| ____|  _ \  __      _(_)_ __ ___| | ___  ___ ___ 
 *   / _ \ | |\/| |  _ \|  _| | |_) | \ \ /\ / / | '__/ _ \ |/ _ \/ __/ __|
 *  / ___ \| |  | | |_) | |___|  _ <   \ V  V /| | | |  __/ |  __/\__ \__ \
 * /_/   \_\_|  |_|____/|_____|_| \_\   \_/\_/ |_|_|  \___|_|\___||___/___/
 *
 *
 * Module Name:
 * ============
 *
 *			LMX_Std.h
 *
 * Purpose:			
 * ========
 *
 *                      Global Settings for Bluetooth and LMX9820
 *
 * Author(s) & Creation Date:	
 * ==========================
 *
 *			Timo Koch, 2004/03/01
 *
 *
 * Copyright:	
 * ==========
 *
 *			AMBER wireless GmbH, Inc.
 *			Albin Koebis Strasse 18
 *			D-50858 Koeln, Germany
 *			Phone:		+49 (0) 2203-369472
 *			Fax:		+49 (0) 2203-459883
 *			Internet:	www.amber-wireless.de
 *
 *----------------------------------------------------------------
 * All software and related documentation is provided "AS IS" and 
 * without warranty or support of any kind and AMBER wireless 
 * expressly disclaims all other warranties, express or implied, 
 * including, but not limited to, the implied warranties of 
 * merchantability and fitness for a particular purpose. Under no 
 * circumstances shall AMBER wireless be liable for any 
 * incidental, special or consequential damages that result from 
 * the use or inability to use the software or related 
 * documentation, even if AMBER wireless has been advised of 
 * the liability.
 *
 * Unless otherwise stated, software written and copyrighted by 
 * AMBER wireless is distributed as "freeware". You may use 
 * and modify this software without any charge or restriction. 
 * You may distribute to others, as long as the original author 
 * is acknowledged.
 *
 * You may not use the Program in non-AMBER devices.
 *
 *****************************************************************************
 *****************************************************************************/
#ifndef _LMX_STD_H_
#define _LMX_STD_H_

#include "MY_std.h"


/*****************************************************************************
 * Global									    						     *
 *****************************************************************************/

#define MAX_PACKET_LENGHT   150       // Max Packet Lenght = 333 bytes  
#define MAX_SEARCH_PATTERN  85 * 2    // Max Number of 16-bit UUIDs in Search pattern    
#define MAX_ATTRIBUTES      85 * 2    // Max List-Size of requested attributes 
#define NO_OF_FORMATS       7         // Max Number of Formats  = 7 bytes
#define NO_OF_DATA_STORES   5         // Max Number of Data Stores = 5 bytes 
#define DEVICE_NAME_LENGHT  30        // Max 30 bytes including NULL termination
#define PIN_LENGHT          16        // Max 16 bytes (128 bits) Pincode-length

#define FRAME_OVERHEAD          7       // Overhead of Frames STX,ETX etc.

// Start delimiter
#define STX     0x02

//Packet type identification
#define REQ     0x52
#define CFM     0x43
#define IND     0x69
#define RES     0x72

// End delimiter
#define ETX     0x03


#ifndef _BD_ADDR_T_
#define _BD_ADDR_T_
typedef struct
{
  uint8_t  Upper_Part_MSB;
  uint8_t  Upper_Part_LSB;
  uint8_t  Middle_Part_MSB;
  uint8_t  Middle_Part_LSB;
  uint8_t  Lower_Part_MSB;
  uint8_t  Lower_Part_LSB;
  
}BD_ADDR_t;
#endif




#ifndef _DEFAULT_CONNECTIONS_T_
#define _DEFAULT_CONNECTIONS_T_
typedef struct 
{
  uint8_t EntryStatus;
  BD_ADDR_t     RemoteBdAddress;
  uint8_t LocalPort;
  uint8_t RemotePort; 
}DEFAULT_CONNECTIONS_t;
#endif


#ifndef _LMX_SYSTEM_PARAMETERS_T_
#define _LMX_SYSTEM_PARAMETERS_T_
typedef struct 
{
  BD_ADDR_t             BdAddr;                 // Bluetooth Device Address 
  uint8_t         Device_Name[30];        // Friendly Device Name max 30 Bytes
  uint8_t         Pincode[16];            // Pin Code default 0000 max 16 Bytes
  uint32_t         Class_of_Device;        // Device Class default 000000
  uint32_t         SPP_Ports_to_Open;     
  DEFAULT_CONNECTIONS_t Default_Connections[3]; // Max 3 Connections on default
  uint8_t         Preferred_Master;       // Forces Slave/Master Switch 
  uint8_t         UART_Speed;             // Default Uart Speed
  uint8_t         UART_Parity;            // Parity Setting
  uint8_t         UART_No_of_Stopbits;    // Stopbit Setting
  uint8_t         Operation_Mode;         // Operation Mode
  uint8_t         Inquiry_Scan_Mode;      // Sets Discoverability of the Device
  uint8_t         Page_Scan_Mode;         // Sets Connectability of the Device
  uint8_t         Security_Mode;          // current security mode of the Bluetooth device.
  uint8_t         Event_Filter;
  uint16_t          Link_Policy;        

}LMX_System_Parameters_t;
#endif



#ifndef _LMX_COMMAND_PROTOCOL_T_
#define _LMX_COMMAND_PROTOCOL_T_
typedef struct
{
  uint8_t Start_delimiter;
  uint8_t Packet_Type_identification;
  uint8_t Op_code;
  uint8_t Data_length_MSB;
  uint8_t Data_length_LSB;
  uint8_t Checksum;
  uint8_t Packet_Data[MAX_PACKET_LENGHT];
  uint8_t End_delimiter;

}LMX_Command_Protocol_t;
#endif


#endif
