/*****************************************************************************
 *****************************************************************************
 *     _    __  __ ____  _____ ____             _          _               
 *    / \  |  \/  | __ )| ____|  _ \  __      _(_)_ __ ___| | ___  ___ ___ 
 *   / _ \ | |\/| |  _ \|  _| | |_) | \ \ /\ / / | '__/ _ \ |/ _ \/ __/ __|
 *  / ___ \| |  | | |_) | |___|  _ <   \ V  V /| | | |  __/ |  __/\__ \__ \
 * /_/   \_\_|  |_|____/|_____|_| \_\   \_/\_/ |_|_|  \___|_|\___||___/___/
 *
 *
 * Module Name:
 * ============
 *
 *			SPP_Commands.C
 *
 * Purpose:			
 * ========
 *
 *			Provides Functions for SPP Commands
 *
 * Author(s) & Creation Date:	
 * ==========================
 *
 *			Timo Koch, 2004/03/01
 *
 *
 * Copyright:	
 * ==========
 *
 *			AMBER wireless GmbH, Inc.
 *			Albin Koebis Strasse 18
 *			D-50858 Koeln, Germany
 *			Phone:		+49 (0) 2203-369472
 *			Fax:		+49 (0) 2203-459883
 *			Internet:	www.amber-wireless.de	
 *
 *----------------------------------------------------------------
 * All software and related documentation is provided "AS IS" and 
 * without warranty or support of any kind and AMBER wireless 
 * expressly disclaims all other warranties, express or implied, 
 * including, but not limited to, the implied warranties of 
 * merchantability and fitness for a particular purpose. Under no 
 * circumstances shall AMBER wireless be liable for any 
 * incidental, special or consequential damages that result from 
 * the use or inability to use the software or related 
 * documentation, even if AMBER wireless has been advised of 
 * the liability.
 *
 * Unless otherwise stated, software written and copyrighted by 
 * AMBER wireless is distributed as "freeware". You may use 
 * and modify this software without any charge or restriction. 
 * You may distribute to others, as long as the original author 
 * is acknowledged.
 *
 * You may not use the Program in non-AMBER devices.
 *
 *****************************************************************************
 *****************************************************************************/

#include "SPP_Commands.h"


/*****************************************************************************
 * Macros & Typedefs							     *
 *****************************************************************************/

/********************************
 * Constant macros		*
 ********************************/

/********************************
 * Function macros		*
 ********************************/

/********************************
 * Typedefs			*
 ********************************/

/*****************************************************************************
 * Static / Non-static global declarations				     *
 *****************************************************************************/

/********************************
 * Non-static globals 		*
 ********************************/

/********************************
 * Static globals		*
 ********************************/

/********************************
 * Function declarations	*
 ********************************/

uint8_t SPP_Establish_Link_REQ       (SPP_ESTABLISH_LINK_REQ_t* SPP_Request);
uint8_t SPP_Send_Data_REQ            (SPP_SEND_DATA_REQ_t* SPP_Request);
uint8_t SPP_Realease_Link_REQ        (SPP_RELEASE_LINK_REQ_t* SPP_Request);
uint8_t SPP_Transparent_Mode_REQ     (SPP_TRANSPARENT_MODE_REQ_t* SPP_Request);
uint8_t SPP_Store_Default_Con_REQ    (SPP_STORE_DEFAULT_CON_REQ_t* SPP_Request);
uint8_t SPP_Connect_Default_Con_REQ  (SPP_CONNECT_DEFAULT_CON_REQ_t* SPP_Request);
uint8_t SPP_Get_List_Default_Con_REQ (SPP_GET_LIST_DEFAULT_CON_REQ_t* SPP_Request);
uint8_t SPP_Delete_Default_Con_REQ   (SPP_DELETE_DEFAULT_CON_REQ_t* SPP_Request);
uint8_t SPP_Set_Port_Config_REQ      (SPP_SET_PORT_CONFIG_REQ_t* SPP_Request);
uint8_t SPP_Get_Port_Config_REQ      (SPP_GET_PORT_CONFIG_REQ_t* SPP_Request);
uint8_t SPP_Get_Port_Status_REQ      (SPP_GET_PORT_STATUS_REQ_t* SPP_Request);
uint8_t SPP_Port_Set_DTR_REQ         (SPP_PORT_SET_DTR_REQ_t* SPP_Request);
uint8_t SPP_Port_Set_RTS_REQ         (SPP_PORT_SET_RTS_REQ_t* SPP_Request);
uint8_t SPP_Port_Break_REQ           (SPP_PORT_BREAK_REQ_t* SPP_Request);
uint8_t SPP_Port_Overrun_Error_REQ   (SPP_PORT_OVERRUN_ERROR_REQ_t* SPP_Request);
uint8_t SPP_Port_Parity_Error_REQ    (SPP_PORT_PARITY_ERROR_REQ_t* SPP_Request);
uint8_t SPP_Port_Framing_Error_REQ   (SPP_PORT_FRAMING_ERROR_REQ_t* SPP_Request);



/*****************************************************************************
 * Function definitions							     * 
 *****************************************************************************/

/*****************************************************************************
 * SPP Routines 							     * 
 *****************************************************************************/
uint8_t SPP_Establish_Link_REQ (SPP_ESTABLISH_LINK_REQ_t* SPP_Request)
{
   
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(SPP_ESTABLISH_LINK);                        // Op_code
  UART1_TX_WriteByte(8);                                         // Data_length_LSB
  UART1_TX_WriteByte(0);                                         // Data_length_MSB 
  UART1_TX_WriteByte((uint8_t)(REQ + SPP_ESTABLISH_LINK + 8));      // Checksum
  UART1_TX_WriteByte(SPP_Request->LocalPort);                    // Data
  UART1_TX_WriteByte(SPP_Request->BdAddr.Upper_Part_MSB);        // Data
  UART1_TX_WriteByte(SPP_Request->BdAddr.Upper_Part_LSB);        // Data
  UART1_TX_WriteByte(SPP_Request->BdAddr.Middle_Part_MSB);       // Data
  UART1_TX_WriteByte(SPP_Request->BdAddr.Middle_Part_LSB);       // Data
  UART1_TX_WriteByte(SPP_Request->BdAddr.Lower_Part_MSB);        // Data
  UART1_TX_WriteByte(SPP_Request->BdAddr.Lower_Part_LSB);        // Data
  UART1_TX_WriteByte(SPP_Request->RemotePortNumber);             // Data
  UART1_TX_WriteByte(ETX);                                       // End_delimiter
  
  return (1);                                                               // OK
}


uint8_t SPP_Send_Data_REQ (SPP_SEND_DATA_REQ_t* SPP_Request)
{
uint16_t i;
  i = *((uint16_t*)&(SPP_Request->PayloadSize));
 i+=3;
//*((uint8_t*)&(SDAP_Request->BrowseGroupID)+ 0)
  
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(SPP_SEND_DATA);                             // Op_code
  UART1_TX_WriteByte((uint8_t)i);                                   // Data_length_LSB
  UART1_TX_WriteByte((uint8_t)(i>>8));                              // Data_length_MSB 
  UART1_TX_WriteByte((uint8_t)(REQ + SPP_SEND_DATA + i));           // Checksum
  UART1_TX_WriteByte(SPP_Request->LocalPort);                    // Data
  i -= 3;
  UART1_TX_WriteByte((uint8_t)(i)); // Data
  UART1_TX_WriteByte((uint8_t)(i>>8)); // Data
  
  for (i=0; i < SPP_Request->PayloadSize; i++)
    UART1_TX_WriteByte(SPP_Request->PayloadData[i]);             // Data
  
  UART1_TX_WriteByte(ETX);                                       // End_delimiter
  
  return (1);                                                               // OK
}


uint8_t SPP_Realease_Link_REQ (SPP_RELEASE_LINK_REQ_t* SPP_Request)
{
  
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(SPP_RELEASE_LINK);                          // Op_code
  UART1_TX_WriteByte(1);                                         // Data_length_LSB
  UART1_TX_WriteByte(0);                                         // Data_length_MSB 
  UART1_TX_WriteByte((uint8_t)(REQ + SPP_RELEASE_LINK + 1));        // Checksum
  UART1_TX_WriteByte(SPP_Request->LocalPort);                    // Data
  
  UART1_TX_WriteByte(ETX);                                       // End_delimiter
  
  return (1);                                                               // OK
}


uint8_t SPP_Transparent_Mode_REQ (SPP_TRANSPARENT_MODE_REQ_t* SPP_Request)
{
 
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(SPP_TRANSPARENT_MODE);                      // Op_code
  UART1_TX_WriteByte(1);                                         // Data_length_LSB
  UART1_TX_WriteByte(0);                                         // Data_length_MSB 
  UART1_TX_WriteByte((uint8_t)(REQ + SPP_TRANSPARENT_MODE + 1));    // Checksum
  UART1_TX_WriteByte(SPP_Request->LocalPort);                    // Data
  
  UART1_TX_WriteByte(ETX);                                       // End_delimiter
  
  return (1);                                                               // OK
}

uint8_t SPP_Store_Default_Con_REQ (SPP_STORE_DEFAULT_CON_REQ_t* SPP_Request)
{
  
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(SPP_STORE_DEFAULT_CON);                     // Op_code
  UART1_TX_WriteByte(10);                                        // Data_length_LSB
  UART1_TX_WriteByte(0);                                         // Data_length_MSB 
  UART1_TX_WriteByte((uint8_t)(REQ + SPP_STORE_DEFAULT_CON + 10));  // Checksum
  UART1_TX_WriteByte(SPP_Request->Index);                        // Data
  UART1_TX_WriteByte(SPP_Request->LocalPort);                    // Data
  UART1_TX_WriteByte(SPP_Request->RemotePort);                   // Data
  UART1_TX_WriteByte(SPP_Request->RemoteBdAddress.Upper_Part_MSB); // Data
  UART1_TX_WriteByte(SPP_Request->RemoteBdAddress.Upper_Part_LSB); // Data
  UART1_TX_WriteByte(SPP_Request->RemoteBdAddress.Middle_Part_MSB);// Data
  UART1_TX_WriteByte(SPP_Request->RemoteBdAddress.Middle_Part_LSB);// Data
  UART1_TX_WriteByte(SPP_Request->RemoteBdAddress.Lower_Part_MSB); // Data
  UART1_TX_WriteByte(SPP_Request->RemoteBdAddress.Lower_Part_LSB); // Data
  UART1_TX_WriteByte(SPP_Request->TransparentMode);              // Data
  UART1_TX_WriteByte(ETX);                                       // End_delimiter
  
  return (1);                                                               // OK
}


uint8_t SPP_Connect_Default_Con_REQ (SPP_CONNECT_DEFAULT_CON_REQ_t* SPP_Request)
{
   
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(SPP_CONNECT_DEFAULT_CON);                   // Op_code
  UART1_TX_WriteByte(1);                                         // Data_length_LSB
  UART1_TX_WriteByte(0);                                         // Data_length_MSB 
  UART1_TX_WriteByte((uint8_t)(REQ + SPP_CONNECT_DEFAULT_CON + 1)); // Checksum
  UART1_TX_WriteByte(SPP_Request->Index);                        // Data
  
  UART1_TX_WriteByte(ETX);                                       // End_delimiter
  
  return (1);                                                               // OK
}


uint8_t SPP_Get_List_Default_Con_REQ (SPP_GET_LIST_DEFAULT_CON_REQ_t* SPP_Request)
{
  
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(SPP_GET_LIST_DEFAULT_CON);                  // Op_code
  UART1_TX_WriteByte(1);                                         // Data_length_LSB
  UART1_TX_WriteByte(0);                                         // Data_length_MSB 
  UART1_TX_WriteByte((uint8_t)(REQ + SPP_GET_LIST_DEFAULT_CON + 1));// Checksum
  UART1_TX_WriteByte(SPP_Request->Index);                        // Data
  
  UART1_TX_WriteByte(ETX);                                       // End_delimiter
  
  return (1);                                                               // OK
}


uint8_t SPP_Delete_Default_Con_REQ (SPP_DELETE_DEFAULT_CON_REQ_t* SPP_Request)
{
  
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(SPP_DELETE_DEFAULT_CON);                    // Op_code
  UART1_TX_WriteByte(1);                                         // Data_length_LSB
  UART1_TX_WriteByte(0);                                         // Data_length_MSB 
  UART1_TX_WriteByte((uint8_t)(REQ + SPP_DELETE_DEFAULT_CON + 1));  // Checksum
  UART1_TX_WriteByte(SPP_Request->Index);                        // Data
  
  UART1_TX_WriteByte(ETX);                                       // End_delimiter
  
  return (1);                                                               // OK
}


uint8_t SPP_Set_Port_Config_REQ (SPP_SET_PORT_CONFIG_REQ_t* SPP_Request)
{
  
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(SPP_SET_PORT_CONFIG);                       // Op_code
  UART1_TX_WriteByte(6);                                         // Data_length_LSB
  UART1_TX_WriteByte(0);                                         // Data_length_MSB 
  UART1_TX_WriteByte((uint8_t)(REQ + SPP_SET_PORT_CONFIG + 6));     // Checksum
  UART1_TX_WriteByte(SPP_Request->LocalPort);                    // Data
  UART1_TX_WriteByte(SPP_Request->BaudRate);                     // Data
  UART1_TX_WriteByte(SPP_Request->Portsettings);                 // Data
  UART1_TX_WriteByte(SPP_Request->FlowControl);                  // Data
  UART1_TX_WriteByte(SPP_Request->XonChar);                      // Data
  UART1_TX_WriteByte(SPP_Request->XoffChar);                     // Data
  
  UART1_TX_WriteByte(ETX);                                       // End_delimiter
  
  return (1);                                                               // OK
}


uint8_t SPP_Get_Port_Config_REQ (SPP_GET_PORT_CONFIG_REQ_t* SPP_Request)
{
 
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(SPP_GET_PORT_CONFIG);                       // Op_code
  UART1_TX_WriteByte(1);                                         // Data_length_LSB
  UART1_TX_WriteByte(0);                                         // Data_length_MSB 
  UART1_TX_WriteByte((uint8_t)(REQ + SPP_GET_PORT_CONFIG + 1));     // Checksum
  UART1_TX_WriteByte(SPP_Request->LocalPort);                    // Data
    
  UART1_TX_WriteByte(ETX);                                       // End_delimiter
  
  return (1);                                                               // OK
}



uint8_t SPP_Get_Port_Status_REQ (SPP_GET_PORT_STATUS_REQ_t* SPP_Request)
{
  
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(SPP_GET_PORT_STATUS);                       // Op_code
  UART1_TX_WriteByte(1);                                         // Data_length_LSB
  UART1_TX_WriteByte(0);                                         // Data_length_MSB 
  UART1_TX_WriteByte((uint8_t)(REQ + SPP_GET_PORT_STATUS + 1));     // Checksum
  UART1_TX_WriteByte(SPP_Request->LocalPort);                    // Data
    
  UART1_TX_WriteByte(ETX);                                       // End_delimiter
  
  return (1);                                                               // OK
}



uint8_t SPP_Port_Set_DTR_REQ (SPP_PORT_SET_DTR_REQ_t* SPP_Request)
{
  
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(SPP_PORT_SET_DTR);                          // Op_code
  UART1_TX_WriteByte(2);                                         // Data_length_LSB
  UART1_TX_WriteByte(0);                                         // Data_length_MSB 
  UART1_TX_WriteByte((uint8_t)(REQ + SPP_PORT_SET_DTR + 2));        // Checksum
  UART1_TX_WriteByte(SPP_Request->LocalPort);                    // Data
  UART1_TX_WriteByte(SPP_Request->State);                        // Data
    
  UART1_TX_WriteByte(ETX);                                       // End_delimiter
  
  return (1);                                                               // OK
}


uint8_t SPP_Port_Set_RTS_REQ (SPP_PORT_SET_RTS_REQ_t* SPP_Request)
{
  
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(SPP_PORT_SET_RTS);                          // Op_code
  UART1_TX_WriteByte(2);                                         // Data_length_LSB
  UART1_TX_WriteByte(0);                                         // Data_length_MSB 
  UART1_TX_WriteByte((uint8_t)(REQ + SPP_PORT_SET_RTS + 2));        // Checksum
  UART1_TX_WriteByte(SPP_Request->LocalPort);                    // Data
  UART1_TX_WriteByte(SPP_Request->State);                        // Data
    
  UART1_TX_WriteByte(ETX);                                       // End_delimiter
  
  return (1);                                                               // OK
}


uint8_t SPP_Port_Break_REQ (SPP_PORT_BREAK_REQ_t* SPP_Request)
{
  
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(SPP_PORT_BREAK);                            // Op_code
  UART1_TX_WriteByte(2);                                         // Data_length_LSB
  UART1_TX_WriteByte(0);                                         // Data_length_MSB 
  UART1_TX_WriteByte((uint8_t)(REQ + SPP_PORT_BREAK + 2));          // Checksum
  UART1_TX_WriteByte(SPP_Request->LocalPort);                    // Data
  UART1_TX_WriteByte(SPP_Request->BreakLength);                  // Data
    
  UART1_TX_WriteByte(ETX);                                       // End_delimiter
  
  return (1);                                                               // OK
}



uint8_t SPP_Port_Overrun_Error_REQ (SPP_PORT_OVERRUN_ERROR_REQ_t* SPP_Request)
{
  
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(SPP_PORT_OVERRUN_ERROR);                    // Op_code
  UART1_TX_WriteByte(1);                                         // Data_length_LSB
  UART1_TX_WriteByte(0);                                         // Data_length_MSB 
  UART1_TX_WriteByte((uint8_t)(REQ + SPP_PORT_OVERRUN_ERROR + 1));  // Checksum
  UART1_TX_WriteByte(SPP_Request->LocalPort);                    // Data
      
  UART1_TX_WriteByte(ETX);                                       // End_delimiter
  
  return (1);                                                               // OK
}


uint8_t SPP_Port_Parity_Error_REQ (SPP_PORT_PARITY_ERROR_REQ_t* SPP_Request)
{
  
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(SPP_PORT_PARITY_ERROR);                     // Op_code
  UART1_TX_WriteByte(1);                                         // Data_length_LSB
  UART1_TX_WriteByte(0);                                         // Data_length_MSB 
  UART1_TX_WriteByte((uint8_t)(REQ + SPP_PORT_PARITY_ERROR + 1));   // Checksum
  UART1_TX_WriteByte(SPP_Request->LocalPort);                    // Data
      
  UART1_TX_WriteByte(ETX);                                       // End_delimiter
  
  return (1);                                                               // OK
}


uint8_t SPP_Port_Framing_Error_REQ (SPP_PORT_FRAMING_ERROR_REQ_t* SPP_Request)
{
  
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(SPP_PORT_FRAMING_ERROR);                    // Op_code
  UART1_TX_WriteByte(1);                                         // Data_length_LSB
  UART1_TX_WriteByte(0);                                         // Data_length_MSB 
  UART1_TX_WriteByte((uint8_t)(REQ + SPP_PORT_FRAMING_ERROR + 1));  // Checksum
  UART1_TX_WriteByte(SPP_Request->LocalPort);                    // Data
      
  UART1_TX_WriteByte(ETX);                                       // End_delimiter
  
  return (1);                                                               // OK
}


