/*****************************************************************************
 *****************************************************************************
 *     _    __  __ ____  _____ ____             _          _               
 *    / \  |  \/  | __ )| ____|  _ \  __      _(_)_ __ ___| | ___  ___ ___ 
 *   / _ \ | |\/| |  _ \|  _| | |_) | \ \ /\ / / | '__/ _ \ |/ _ \/ __/ __|
 *  / ___ \| |  | | |_) | |___|  _ <   \ V  V /| | | |  __/ |  __/\__ \__ \
 * /_/   \_\_|  |_|____/|_____|_| \_\   \_/\_/ |_|_|  \___|_|\___||___/___/
 *
 *																			 
 * Module Name:																  
 * ============
 *
 *			SPP_Commands.h
 *
 * Purpose:			
 * ========
 *
 *                      Header File for SPP_Commands.c
 *			
 * Author(s) & Creation Date:	
 * ==========================
 *
 *			Timo Koch, 2004/03/01
 *
 *
 * Copyright:	
 * ==========
 *
 *			AMBER wireless GmbH, Inc.
 *			Albin Koebis Strasse 18
 *			D-50858 Koeln, Germany
 *			Phone:		+49 (0) 2203-369472
 *			Fax:		+49 (0) 2203-459883
 *			Internet:	www.amber-wireless.de
 *
 *----------------------------------------------------------------
 * All software and related documentation is provided "AS IS" and 
 * without warranty or support of any kind and AMBER wireless 
 * expressly disclaims all other warranties, express or implied, 
 * including, but not limited to, the implied warranties of 
 * merchantability and fitness for a particular purpose. Under no 
 * circumstances shall AMBER wireless be liable for any 
 * incidental, special or consequential damages that result from 
 * the use or inability to use the software or related 
 * documentation, even if AMBER wireless has been advised of 
 * the liability.
 *
 * Unless otherwise stated, software written and copyrighted by 
 * AMBER wireless is distributed as "freeware". You may use 
 * and modify this software without any charge or restriction. 
 * You may distribute to others, as long as the original author 
 * is acknowledged.
 *
 * You may not use the Program in non-AMBER devices.
 *
 *****************************************************************************
 *****************************************************************************/




#ifndef _SPP_COMMANDS_H_
#define _SPP_COMMANDS_H_

#include "MY_std.h"
#include "LMX_Std.h"
#include "main.h"

/*****************************************************************************
 * Macros & Typedefs							     *
 *****************************************************************************/


/********************************
 * Constant macros		*
 ********************************/




// Opcodes

#define SPP_SET_PORT_CONFIG             0x07
#define SPP_GET_PORT_CONFIG             0x08
#define SPP_PORT_CONFIG_CHANGED         0x09
#define SPP_ESTABLISH_LINK              0x0A
#define SPP_LINK_ESTABLISHED            0x0B
#define SPP_INCOMMING_LINK_ESTABLISHED  0x0C
#define SPP_RELEASE_LINK                0x0D
#define SPP_LINK_RELEASED               0x0E
#define SPP_SEND_DATA                   0x0F
#define SPP_INCOMING_DATA               0x10
#define SPP_TRANSPARENT_MODE            0x11
#define SPP_CONNECT_DEFAULT_CON         0x12
#define SPP_STORE_DEFAULT_CON           0x13
#define SPP_GET_LIST_DEFAULT_CON        0x14
#define SPP_DELETE_DEFAULT_CON          0x15
#define SPP_PORT_STATUS_CHANGED         0x3E
#define SPP_GET_PORT_STATUS             0x40
#define SPP_PORT_SET_DTR                0x41
#define SPP_PORT_SET_RTS                0x42
#define SPP_PORT_BREAK                  0x43
#define SPP_PORT_OVERRUN_ERROR          0x44
#define SPP_PORT_PARITY_ERROR           0x45
#define SPP_PORT_FRAMING_ERROR          0x46


// Msg identification ( OP-Code + Packet type)
// easier and safer handle of incoming messages
/*
Packet type identification
#define REQ     0x52
#define CFM     0x43
#define IND     0x69
#define RES     0x72
*/

#define SPP_SET_PORT_CONFIG_REQ             0x0752
#define SPP_SET_PORT_CONFIG_CFM             0x0743
#define SPP_GET_PORT_CONFIG_REQ             0x0852
#define SPP_GET_PORT_CONFIG_CFM             0x0843
#define SPP_PORT_CONFIG_CHANGED_IND         0x0969
#define SPP_ESTABLISH_LINK_REQ              0x0A52
#define SPP_ESTABLISH_LINK_CFM              0x0A43
#define SPP_LINK_ESTABLISHED_IND            0x0B69
#define SPP_INCOMMING_LINK_ESTABLISHED_IND  0x0C69
#define SPP_RELEASE_LINK_REQ                0x0D52
#define SPP_RELEASE_LINK_CFM                0x0D43
#define SPP_LINK_RELEASED_IND               0x0E69
#define SPP_SEND_DATA_REQ                   0x0F52
#define SPP_SEND_DATA_CFM                   0x0F43
#define SPP_INCOMING_DATA_IND               0x1069
#define SPP_TRANSPARENT_MODE_REQ            0x1152
#define SPP_TRANSPARENT_MODE_CFM            0x1143
#define SPP_TRANSPARENT_MODE_IND            0x1169
#define SPP_CONNECT_DEFAULT_CON_REQ         0x1252
#define SPP_CONNECT_DEFAULT_CON_CFM         0x1243
#define SPP_STORE_DEFAULT_CON_REQ           0x1352
#define SPP_STORE_DEFAULT_CON_CFM           0x1343
#define SPP_GET_LIST_DEFAULT_CON_REQ        0x1452
#define SPP_GET_LIST_DEFAULT_CON_CFM        0x1443
#define SPP_DELETE_DEFAULT_CON_REQ          0x1552
#define SPP_DELETE_DEFAULT_CON_CFM          0x1543
#define SPP_PORT_STATUS_CHANGED_IND         0x3E69
#define SPP_GET_PORT_STATUS_REQ             0x4052
#define SPP_GET_PORT_STATUS_CFM             0x4043
#define SPP_PORT_SET_DTR_REQ                0x4152
#define SPP_PORT_SET_DTR_CFM                0x4143
#define SPP_PORT_SET_RTS_REQ                0x4252
#define SPP_PORT_SET_RTS_CFM                0x4243
#define SPP_PORT_BREAK_REQ                  0x4352
#define SPP_PORT_BREAK_CFM                  0x4343
#define SPP_PORT_OVERRUN_ERROR_REQ          0x4452
#define SPP_PORT_OVERRUN_ERROR_CFM          0x4443
#define SPP_PORT_PARITY_ERROR_REQ           0x4552
#define SPP_PORT_PARITY_ERROR_CFM           0x4543
#define SPP_PORT_FRAMING_ERROR_REQ          0x4652
#define SPP_PORT_FRAMING_ERROR_CFM          0x4643


/********************************
 * Function macros		*
 ********************************/

/********************************
 * Typedefs			*
 ********************************/
 
/********************************
 * Establish Link               *
 ********************************/
typedef struct
{
  uint8_t LocalPort;
  BD_ADDR_t     BdAddr;
  uint8_t RemotePortNumber; 
}SPP_ESTABLISH_LINK_REQ_t;

typedef struct 
{
  uint8_t Status;
  uint8_t LocalPort;
}SPP_ESTABLISH_LINK_CFM_t;

typedef struct
{
  uint8_t Status;
  BD_ADDR_t     BdAddr;
  uint8_t LocalPort;
  uint8_t RemotePortNumber;
}SPP_LINK_ESTABLISHED_IND_t;


/********************************
 * SPP Release Link             *
 ********************************/
typedef struct
{
  uint8_t LocalPort;
}SPP_RELEASE_LINK_REQ_t;

typedef struct
{
  uint8_t Status;
  uint8_t LocalPort;
}SPP_RELEASE_LINK_CFM_t;

typedef struct
{
  uint8_t Reason;
  uint8_t LocalPort;
}SPP_LINK_RELEASED_IND_t;


/********************************
 * SPP Send Data                *
 ********************************/
typedef struct
{
  uint8_t LocalPort;
  uint16_t  PayloadSize;
  uint8_t PayloadData[MAX_PACKET_LENGHT];
   
}SPP_SEND_DATA_REQ_t;

typedef struct
{
  uint8_t Status;
  uint8_t LocalPort;
}SPP_SEND_DATA_CFM_t;

typedef struct
{
  uint8_t LocalPort;
  uint8_t PayloadSize_LSB;
  uint8_t PayloadSize_MSB;
  uint8_t PayloadData[MAX_PACKET_LENGHT];
}SPP_INCOMING_DATA_IND_t;


/********************************
 * SPP Transparent Mode         *
 ********************************/
typedef struct
{
  uint8_t LocalPort;
}SPP_TRANSPARENT_MODE_REQ_t;

typedef struct
{
  uint8_t Status;
  uint8_t LocalPort;
}SPP_TRANSPARENT_MODE_CFM_t;

typedef struct
{
  uint8_t LocalPort;
  uint8_t Mode;
}SPP_TRANSPARENT_MODE_IND_t;


/********************************
 * SPP Store Default Connection *
 ********************************/
typedef struct 
{
  uint8_t Index;
  uint8_t LocalPort;
  uint8_t RemotePort;
  BD_ADDR_t     RemoteBdAddress;
  uint8_t TransparentMode;
}SPP_STORE_DEFAULT_CON_REQ_t;

typedef struct
{
  uint8_t Status;
}SPP_STORE_DEFAULT_CON_CFM_t;


/********************************
 * SPP Connect Default Connection *
 ********************************/
typedef struct
{
  uint8_t Index;
}SPP_CONNECT_DEFAULT_CON_REQ_t;


typedef struct 
{
  uint8_t Index;
}SPP_GET_LIST_DEFAULT_CON_REQ_t;


/********************************
 * SPP Delete Default Connection*
 ********************************/
typedef struct
{
  uint8_t Index;
}SPP_DELETE_DEFAULT_CON_REQ_t;

typedef struct
{
  uint8_t Index;
  uint8_t Status;
}SPP_DELETE_DEFAULT_CON_CFM_t;


/********************************
 * SPP Set Port Configuration   *
 ********************************/
typedef struct
{
  uint8_t LocalPort;
  uint8_t BaudRate;
  uint8_t Portsettings;
  uint8_t FlowControl;
  uint8_t XonChar;
  uint8_t XoffChar;
}SPP_SET_PORT_CONFIG_REQ_t;

typedef struct
{
  uint8_t Status;
  uint8_t LocalPort;
}SPP_SET_PORT_CONFIG_CFM_t;


/********************************
 * SPP Get Port Config          *  
 ********************************/
typedef struct
{
  uint8_t LocalPort; 
}SPP_GET_PORT_CONFIG_REQ_t;

typedef struct
{
  uint8_t Status;
  uint8_t LocalPort;
  uint8_t BaudRate;
  uint8_t Portsettings;
  uint8_t FlowControl;
  uint8_t XonChar;
  uint8_t XoffChar;
}SPP_GET_PORT_CONFIG_CFM_t;

typedef struct
{
  uint8_t LocalPort;
  uint8_t BaudRate;
  uint8_t Portsettings;
  uint8_t FlowControl;
  uint8_t XonChar;
  uint8_t XoffChar;
}SPP_PORT_CONFIG_CHANGED_IND_t;


/********************************
 * SPP Get Port Status          *  
 ********************************/
typedef struct
{
  uint8_t LocalPort; 
}SPP_GET_PORT_STATUS_REQ_t;

typedef struct
{
  uint8_t Status;
  uint8_t LocalPort;
  uint8_t PortStatus;
  uint16_t BreakLength;
}SPP_GET_PORT_STATUS_CFM_t;


/********************************
 * SPP Port Set DTR             *  
 ********************************/
typedef struct
{
  uint8_t LocalPort;
  uint8_t State;
}SPP_PORT_SET_DTR_REQ_t;

typedef struct
{
  uint8_t Status;
  uint8_t LocalPort;
}SPP_PORT_SET_DTR_CFM_t;


/********************************
 * SPP Port Set RTS             *  
 ********************************/
typedef struct
{
  uint8_t LocalPort;
  uint8_t State;
}SPP_PORT_SET_RTS_REQ_t;

typedef struct
{
  uint8_t Status;
  uint8_t LocalPort;
}SPP_PORT_SET_RTS_CFM_t;


/********************************
 * SPP Set Port BREAK           *  
 ********************************/
typedef struct
{
  uint8_t LocalPort;
  uint8_t BreakLength;
}SPP_PORT_BREAK_REQ_t;

typedef struct
{
  uint8_t Status;
  uint8_t LocalPort;
}SPP_PORT_BREAK_CFM_t;


/********************************
 * SPP Port Overrun Error       *  
 ********************************/
typedef struct
{
  uint8_t LocalPort; 
}SPP_PORT_OVERRUN_ERROR_REQ_t;

typedef struct
{
  uint8_t Status;
  uint8_t LocalPort;
}SPP_PORT_OVERRUN_ERROR_CFM_t;


/********************************
 * SPP Port Parity Error        *  
 ********************************/
typedef struct
{
  uint8_t LocalPort; 
}SPP_PORT_PARITY_ERROR_REQ_t;

typedef struct
{
  uint8_t Status;
  uint8_t LocalPort;
}SPP_PORT_PARITY_ERROR_CFM_t;


/********************************
 * SPP Port Framing Error       *  
 ********************************/
typedef struct 
{
  uint8_t LocalPort;
}SPP_PORT_FRAMING_ERROR_REQ_t;

typedef struct
{
  uint8_t Status;
  uint8_t LocalPort;
}SPP_PORT_FRAMING_ERROR_CFM_t;

typedef struct
{
  uint8_t LocalPort;
  uint8_t PortStatus;
  uint8_t BreakLength_LSB;
  uint8_t BreakLength_MSB;
}SPP_PORT_STATUS_CHANGED_IND_t;
/*****************************************************************************
 * Exported Variables & Functions					     *
 *****************************************************************************/

/********************************
 * Exported Variables		*
 ********************************/

/********************************
 * Exported Functions		*
 ********************************/

extern uint8_t SPP_Establish_Link_REQ       (SPP_ESTABLISH_LINK_REQ_t*);
extern uint8_t SPP_Send_Data_REQ            (SPP_SEND_DATA_REQ_t*);
extern uint8_t SPP_Realease_Link_REQ        (SPP_RELEASE_LINK_REQ_t*);
extern uint8_t SPP_Transparent_Mode_REQ     (SPP_TRANSPARENT_MODE_REQ_t*);
extern uint8_t SPP_Store_Default_Con_REQ    (SPP_STORE_DEFAULT_CON_REQ_t*);
extern uint8_t SPP_Connect_Default_Con_REQ  (SPP_CONNECT_DEFAULT_CON_REQ_t*);
extern uint8_t SPP_Get_List_Default_Con_REQ (SPP_GET_LIST_DEFAULT_CON_REQ_t*);
extern uint8_t SPP_Delete_Default_Con_REQ   (SPP_DELETE_DEFAULT_CON_REQ_t*);
extern uint8_t SPP_Set_Port_Config_REQ      (SPP_SET_PORT_CONFIG_REQ_t*);
extern uint8_t SPP_Get_Port_Config_REQ      (SPP_GET_PORT_CONFIG_REQ_t*);
extern uint8_t SPP_Get_Port_Status_REQ      (SPP_GET_PORT_STATUS_REQ_t*);
extern uint8_t SPP_Port_Set_DTR_REQ         (SPP_PORT_SET_DTR_REQ_t*);
extern uint8_t SPP_Port_Set_RTS_REQ         (SPP_PORT_SET_RTS_REQ_t*);
extern uint8_t SPP_Port_Break_REQ           (SPP_PORT_BREAK_REQ_t*);
extern uint8_t SPP_Port_Overrun_Error_REQ   (SPP_PORT_OVERRUN_ERROR_REQ_t*);
extern uint8_t SPP_Port_Parity_Error_REQ    (SPP_PORT_PARITY_ERROR_REQ_t*);
extern uint8_t SPP_Port_Framing_Error_REQ   (SPP_PORT_FRAMING_ERROR_REQ_t*);


#endif /* #ifdef _SPP_COMMANDS_H_ */
