/*****************************************************************************
 *****************************************************************************
 *     _    __  __ ____  _____ ____             _          _               
 *    / \  |  \/  | __ )| ____|  _ \  __      _(_)_ __ ___| | ___  ___ ___ 
 *   / _ \ | |\/| |  _ \|  _| | |_) | \ \ /\ / / | '__/ _ \ |/ _ \/ __/ __|
 *  / ___ \| |  | | |_) | |___|  _ <   \ V  V /| | | |  __/ |  __/\__ \__ \
 * /_/   \_\_|  |_|____/|_____|_| \_\   \_/\_/ |_|_|  \___|_|\___||___/___/
 *
 *
 * Module Name:
 * ============
 *
 *			LMX_Commands.C
 *
 * Purpose:			
 * ========
 *
 *			Provide functions for LMX access - OK
 *
 * Author(s) & Creation Date:	
 * ==========================
 *
 *			Timo Koch, 2004/09/18
 *
 *
 * Copyright:	
 * ==========
 *
 *			AMBER wireless GmbH, Inc.
 *			Albin Koebis Strasse 18
 *			D-50858 Koeln, Germany
 *			Phone:		+49 (0) 2203-369472
 *			Fax:		+49 (0) 2203-459883
 *			Internet:	www.amber-wireless.de	
 *
 *----------------------------------------------------------------
 * All software and related documentation is provided "AS IS" and 
 * without warranty or support of any kind and AMBER wireless 
 * expressly disclaims all other warranties, express or implied, 
 * including, but not limited to, the implied warranties of 
 * merchantability and fitness for a particular purpose. Under no 
 * circumstances shall AMBER wireless be liable for any 
 * incidental, special or consequential damages that result from 
 * the use or inability to use the software or related 
 * documentation, even if AMBER wireless has been advised of 
 * the liability.
 *
 * Unless otherwise stated, software written and copyrighted by 
 * AMBER wireless is distributed as "freeware". You may use 
 * and modify this software without any charge or restriction. 
 * You may distribute to others, as long as the original author 
 * is acknowledged.
 *
 * You may not use the Program in non-AMBER devices.
 *
 *****************************************************************************
 *****************************************************************************/

#include "LMX_Commands.h"

/*****************************************************************************
 * Macros & Typedefs							     *
 *****************************************************************************/

/********************************
 * Constant macros		*
 ********************************/

/********************************
 * Function macros		*
 ********************************/

/********************************
 * Typedefs			*
 ********************************/

/*****************************************************************************
 * Static / Non-static global declarations				     *
 *****************************************************************************/

/********************************
 * Non-static globals 		*
 ********************************/

/********************************
 * Static globals		*
 ********************************/



/********************************
 * Function declarations	*
 ********************************/
uint8_t Set_Default_Link_Policy_REQ  (SET_DEFAULT_LINK_POLICY_REQ_t* Request);
uint8_t Get_Default_Link_Policy_REQ  (void);
uint8_t Disable_TL_REQ               (void);
uint8_t Set_Ports_to_open_REQ        (SET_PORTS_TO_OPEN_REQ_t* Request);
uint8_t Get_Ports_to_open_REQ        (void);
uint8_t Enable_SDP_Record_REQ        (ENABLE_SDP_RECORD_REQ_t* Request);
uint8_t Delete_SDP_Records_REQ       (void);
uint8_t Store_SPP_Record_REQ         (STORE_SPP_RECORD_REQ_t* Request);
uint8_t Store_DUN_Record_REQ         (STORE_DUN_RECORD_REQ_t* Request);
uint8_t Store_FAX_Record_REQ         (STORE_FAX_RECORD_REQ_t* Request);
uint8_t Store_OPP_Record_REQ         (STORE_OPP_RECORD_REQ_t* Request);
uint8_t Store_FTP_Record_REQ         (STORE_FTP_RECORD_REQ_t* Request);
uint8_t Store_SYNC_Record_REQ        (STORE_SYNC_RECORD_REQ_t* Request);
uint8_t Change_Local_BdAddr_REQ      (CHANGE_LOCAL_BDADDRESS_REQ_t* Request);
uint8_t Store_Class_of_Device_REQ    (STORE_CLASS_OF_DEVICE_REQ_t* Request);
uint8_t Force_Master_Role_REQ        (FORCE_MASTER_ROLE_REQ_t* Request);
uint8_t Read_Operation_Mode_REQ      (void);
uint8_t Write_Operation_Mode_REQ     (WRITE_OPERATION_MODE_REQ_t* Request);
uint8_t Set_Event_Filter_REQ         (SET_EVENT_FILTER_REQ_t* Request);
uint8_t Get_Event_Filter_REQ         (void);
uint8_t Reset_REQ                    (void);
uint8_t Restore_Factory_Settings_REQ (void);
uint8_t Change_NVS_UART_Speed_REQ    (CHANGE_NVS_UART_SPEED_REQ_t* Request);
uint8_t Change_UART_Settings_REQ     (CHANGE_UART_SETTINGS_REQ_t* Request);
uint8_t Test_Mode_REQ                (TEST_MODE_REQ_t* Request);
uint8_t RF_Test_Mode_REQ             (RF_TEST_MODE_REQ_t* Request);
uint8_t Read_RSSI_REQ                (READ_RSSI_REQ_t* Request);
uint8_t Firmware_Upgrade_REQ         (void);



/*****************************************************************************
 * Function definitions							     * 
 *****************************************************************************/

uint8_t Set_Default_Link_Policy_REQ (SET_DEFAULT_LINK_POLICY_REQ_t* Request)
{

  
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(SET_DEFAULT_LINK_POLICY);                   // Op_code
  UART1_TX_WriteByte(2);                                         // Data_length_LSB 
  UART1_TX_WriteByte(0);                                         // Data_length_MSB
  UART1_TX_WriteByte((uint8_t)(REQ + SET_DEFAULT_LINK_POLICY + 2)); // Checksum
  UART1_TX_WriteByte((uint8_t)(Request->LinkPolicy >> 8));          // Data
  UART1_TX_WriteByte((uint8_t) Request->LinkPolicy);                // Data
  UART1_TX_WriteByte(ETX);                                       // End_delimiter
  
  return (0);                                                               // OK
} 


uint8_t Get_Default_Link_Policy_REQ (void)
{
  
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(GET_DEFAULT_LINK_POLICY);                   // Op_code
  UART1_TX_WriteByte(0);                                         // Data_length_LSB 
  UART1_TX_WriteByte(0);                                         // Data_length_MSB
  UART1_TX_WriteByte((uint8_t)(REQ + GET_DEFAULT_LINK_POLICY));     // Checksum
  UART1_TX_WriteByte(ETX); 

  return(1);                                                                //OK  
} 


uint8_t Disable_TL_REQ (void)
{
  
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(DISABLE_TL);                                // Op_code
  UART1_TX_WriteByte(0);                                         // Data_length_LSB 
  UART1_TX_WriteByte(0);                                         // Data_length_MSB
  UART1_TX_WriteByte((uint8_t)(REQ + DISABLE_TL));                  // Checksum
  UART1_TX_WriteByte(ETX); 

  return(1); 
} 


uint8_t Set_Ports_to_open_REQ (SET_PORTS_TO_OPEN_REQ_t* Request)
{
  
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(SET_PORTS_TO_OPEN);                         // Op_code
  UART1_TX_WriteByte(4);                                         // Data_length_LSB 
  UART1_TX_WriteByte(0);                                         // Data_length_MSB
  UART1_TX_WriteByte((uint8_t)(REQ + SET_PORTS_TO_OPEN + 4));       // Checksum
  UART1_TX_WriteByte((uint8_t)(Request->Ports >> 24));              // Data
  UART1_TX_WriteByte((uint8_t)(Request->Ports >> 16));                     // Data
  UART1_TX_WriteByte((uint8_t)(Request->Ports >> 8));                    // Data
  UART1_TX_WriteByte((uint8_t)(Request->Ports));                    // Data
  UART1_TX_WriteByte(ETX);                                       // End_delimiter
  
  return (1);   
}


uint8_t Get_Ports_to_open_REQ (void)
{
  
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(GET_PORTS_TO_OPEN);                         // Op_code
  UART1_TX_WriteByte(0);                                         // Data_length_LSB 
  UART1_TX_WriteByte(0);                                         // Data_length_MSB
  UART1_TX_WriteByte((uint8_t)(REQ + GET_PORTS_TO_OPEN));           // Checksum
  UART1_TX_WriteByte(ETX); 

  return(1); 
}


uint8_t Enable_SDP_Record_REQ (ENABLE_SDP_RECORD_REQ_t* Request)
{
  
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(ENABLE_SDP_RECORD);                         // Op_code
  UART1_TX_WriteByte(2);                                         // Data_length_LSB 
  UART1_TX_WriteByte(0);                                         // Data_length_MSB
  UART1_TX_WriteByte((uint8_t)(REQ + ENABLE_SDP_RECORD + 2));       // Checksum
  UART1_TX_WriteByte(Request->State);                            // Data
  UART1_TX_WriteByte(Request->Identifier);                       // Data
  UART1_TX_WriteByte(ETX);                                       // End_delimiter
  
  return (1);   
}



uint8_t Delete_SDP_Records_REQ (void)
{
  
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(DELETE_SDP_RECORDS);                        // Op_code
  UART1_TX_WriteByte(0);                                         // Data_length_LSB 
  UART1_TX_WriteByte(0);                                         // Data_length_MSB
  UART1_TX_WriteByte((uint8_t)(REQ + DELETE_SDP_RECORDS));          // Checksum
  UART1_TX_WriteByte(ETX); 

  return(1); 
}


uint8_t Store_SPP_Record_REQ (STORE_SPP_RECORD_REQ_t* Request)
{
uint8_t i; 

  
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(STORE_SPP_RECORD);                          // Op_code
  UART1_TX_WriteByte((4 + Request->NameLength));                 // Data_length_LSB 
  UART1_TX_WriteByte(0);                                         // Data_length_MSB
  UART1_TX_WriteByte((uint8_t)(REQ + STORE_SPP_RECORD + Request->NameLength + 4));// Checksum
  UART1_TX_WriteByte(Request->LocalPort);                        // Data
  UART1_TX_WriteByte(Request->Authentication);                   // Data
  UART1_TX_WriteByte(Request->Encryption);                       // Data
  UART1_TX_WriteByte(Request->NameLength);                       // Data
  
  for (i=0; i < Request->NameLength;i++)
      UART1_TX_WriteByte(Request->ServiceName[i]);     
  
  UART1_TX_WriteByte(ETX);                                       // End_delimiter
  
  return (1);   
}


uint8_t Store_DUN_Record_REQ (STORE_DUN_RECORD_REQ_t* Request)
{
uint8_t i; 
    
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(STORE_DUN_RECORD);                          // Op_code
  UART1_TX_WriteByte((4 + Request->NameLength));                 // Data_length_LSB 
  UART1_TX_WriteByte(0);                                         // Data_length_MSB
  UART1_TX_WriteByte((uint8_t)(REQ + STORE_DUN_RECORD + Request->NameLength + 4));// Checksum
  UART1_TX_WriteByte(Request->LocalPort);                        // Data
  UART1_TX_WriteByte(Request->Authentication);                   // Data
  UART1_TX_WriteByte(Request->Encryption);                       // Data
  UART1_TX_WriteByte(Request->NameLength);                       // Data
  
  for (i=0; i < Request->NameLength;i++)
      UART1_TX_WriteByte(Request->ServiceName[i]);     
  
  UART1_TX_WriteByte(ETX);                                       // End_delimiter
  
  return (1);   
}


uint8_t Store_FAX_Record_REQ (STORE_FAX_RECORD_REQ_t* Request)
{
uint8_t i; 
  
  i = Request->NameLength + 7;
  
    
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(STORE_FAX_RECORD);                          // Op_code
  UART1_TX_WriteByte((uint8_t)i);                                   // Data_length_LSB 
  UART1_TX_WriteByte((uint8_t)(i >> 8));                            // Data_length_MSB
  UART1_TX_WriteByte((uint8_t)(REQ + STORE_FAX_RECORD + i));        // Checksum
  UART1_TX_WriteByte(Request->LocalPort);                        // Data
  UART1_TX_WriteByte(Request->FaxClass1);                        // Data
  UART1_TX_WriteByte(Request->FaxClass20);                       // Data
  UART1_TX_WriteByte(Request->FaxClass2);                        // Data
  UART1_TX_WriteByte(Request->Authentication);                   // Data
  UART1_TX_WriteByte(Request->Encryption);                       // Data
  UART1_TX_WriteByte(Request->NameLength);                       // Data
  
  for (i=0; i < Request->NameLength;i++)
      UART1_TX_WriteByte(Request->ServiceName[i]);     
  
  UART1_TX_WriteByte(ETX);                                       // End_delimiter
  
  return (1);   
}



uint8_t Store_OPP_Record_REQ (STORE_OPP_RECORD_REQ_t* Request)
{
uint8_t i; 
  
  i = Request->NameLength + 5 + Request->NoOfFormats;
  
   
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(STORE_OPP_RECORD);                          // Op_code
  UART1_TX_WriteByte((uint8_t)i);                                   // Data_length_LSB 
  UART1_TX_WriteByte((uint8_t)(i >> 8));                            // Data_length_MSB
  UART1_TX_WriteByte((uint8_t)(REQ + STORE_OPP_RECORD + i));        // Checksum
  UART1_TX_WriteByte(Request->LocalPort);                        // Data
  UART1_TX_WriteByte(Request->Authentication);                   // Data
  UART1_TX_WriteByte(Request->Encryption);                       // Data
  UART1_TX_WriteByte(Request->NoOfFormats);                      // Data
  
  for (i=0; i < Request->NoOfFormats;i++)
      UART1_TX_WriteByte(Request->SupportedFormats[i]);     
  
  UART1_TX_WriteByte(Request->NameLength);                       // Data
  
  for (i=0; i < Request->NameLength;i++)
      UART1_TX_WriteByte(Request->ServiceName[i]);     
   
  UART1_TX_WriteByte(ETX);                                       // End_delimiter
  
  return (1);   
}




uint8_t Store_FTP_Record_REQ (STORE_FTP_RECORD_REQ_t* Request)
{
uint8_t i; 
  i = Request->NameLength + 4;
  
    
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(STORE_FTP_RECORD);                          // Op_code
  UART1_TX_WriteByte((uint8_t)i);                                   // Data_length_LSB 
  UART1_TX_WriteByte((uint8_t)(i >> 8));                            // Data_length_MSB
  UART1_TX_WriteByte((uint8_t)(REQ + STORE_FTP_RECORD + i));        // Checksum
  UART1_TX_WriteByte(Request->LocalPort);                        // Data
  UART1_TX_WriteByte(Request->Authentication);                   // Data
  UART1_TX_WriteByte(Request->Encryption);                       // Data
  UART1_TX_WriteByte(Request->NameLength);                       // Data
    
  for (i=0; i < Request->NameLength;i++)
      UART1_TX_WriteByte(Request->ServiceName[i]);     
   
  UART1_TX_WriteByte(ETX);                                       // End_delimiter
  
  return (1);   
}



uint8_t Store_SYNC_Record_REQ (STORE_SYNC_RECORD_REQ_t* Request)
{
uint8_t i; 
  i = Request->NameLength + 5 + Request->NoOfDataStores;
  
   
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(STORE_SYNC_RECORD);                         // Op_code
  UART1_TX_WriteByte((uint8_t)i);                                   // Data_length_LSB 
  UART1_TX_WriteByte((uint8_t)(i >> 8));                            // Data_length_MSB
  UART1_TX_WriteByte((uint8_t)(REQ + STORE_SYNC_RECORD + i));       // Checksum
  UART1_TX_WriteByte(Request->LocalPort);                        // Data
  UART1_TX_WriteByte(Request->Authentication);                   // Data
  UART1_TX_WriteByte(Request->Encryption);                       // Data
  UART1_TX_WriteByte(Request->NoOfDataStores);                   // Data
    
  for (i=0; i < Request->NoOfDataStores;i++)
      UART1_TX_WriteByte(Request->SupportedDataStores[i]);   
  
  UART1_TX_WriteByte(Request->NameLength);                       // Data
      
  for (i=0; i < Request->NameLength;i++)
      UART1_TX_WriteByte(Request->ServiceName[i]);       
   
  UART1_TX_WriteByte(ETX);                                       // End_delimiter
  
  return (1);   
}


uint8_t Change_Local_BdAddr_REQ (CHANGE_LOCAL_BDADDRESS_REQ_t* Request)
{
   
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(CHANGE_LOCAL_BDADDRESS);                    // Op_code
  UART1_TX_WriteByte(6);                                         // Data_length_LSB 
  UART1_TX_WriteByte(0);                                         // Data_length_MSB
  UART1_TX_WriteByte((uint8_t)(REQ + CHANGE_LOCAL_BDADDRESS + 6));  // Checksum
  UART1_TX_WriteByte(Request->BdAddr.Upper_Part_MSB);            // Data
  UART1_TX_WriteByte(Request->BdAddr.Upper_Part_LSB);            // Data
  UART1_TX_WriteByte(Request->BdAddr.Middle_Part_MSB);           // Data
  UART1_TX_WriteByte(Request->BdAddr.Middle_Part_LSB);           // Data
  UART1_TX_WriteByte(Request->BdAddr.Lower_Part_MSB);            // Data
  UART1_TX_WriteByte(Request->BdAddr.Lower_Part_LSB);            // Data
     
  UART1_TX_WriteByte(ETX);      
  return (1); 
}


uint8_t Store_Class_of_Device_REQ (STORE_CLASS_OF_DEVICE_REQ_t* Request)
{

   
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(STORE_CLASS_OF_DEVICE);                     // Op_code
  UART1_TX_WriteByte(3);                                         // Data_length_LSB 
  UART1_TX_WriteByte(0);                                         // Data_length_MSB
  UART1_TX_WriteByte((uint8_t)(REQ + STORE_CLASS_OF_DEVICE + 3));   // Checksum
  UART1_TX_WriteByte((uint8_t)(Request->ClassOfDevice>>16));        // Data
  UART1_TX_WriteByte((uint8_t)(Request->ClassOfDevice>>8));         // Data
  UART1_TX_WriteByte((uint8_t)(Request->ClassOfDevice));            // Data
       
  UART1_TX_WriteByte(ETX);
  return (1);  
}



uint8_t Force_Master_Role_REQ (FORCE_MASTER_ROLE_REQ_t* Request)
{
   
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(FORCE_MASTER_ROLE);                         // Op_code
  UART1_TX_WriteByte(1);                                         // Data_length_LSB 
  UART1_TX_WriteByte(0);                                         // Data_length_MSB
  UART1_TX_WriteByte((uint8_t)(REQ + FORCE_MASTER_ROLE + 1));       // Checksum
  UART1_TX_WriteByte(Request->Role);                             // Data
  
  UART1_TX_WriteByte(ETX);
  return (1);  
}


uint8_t Read_Operation_Mode_REQ (void)
{
   
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(READ_OPERATION_MODE);                       // Op_code
  UART1_TX_WriteByte(0);                                         // Data_length_LSB 
  UART1_TX_WriteByte(0);                                         // Data_length_MSB
  UART1_TX_WriteByte((uint8_t)(REQ + READ_OPERATION_MODE));         // Checksum
  
  UART1_TX_WriteByte(ETX);
  return (1); 
}


uint8_t Write_Operation_Mode_REQ (WRITE_OPERATION_MODE_REQ_t* Request)
{
   
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(WRITE_OPERATION_MODE);                      // Op_code
  UART1_TX_WriteByte(1);                                         // Data_length_LSB 
  UART1_TX_WriteByte(0);                                         // Data_length_MSB
  UART1_TX_WriteByte((uint8_t)(REQ + WRITE_OPERATION_MODE + 1));    // Checksum
  UART1_TX_WriteByte(Request->Mode);                             // Data
  UART1_TX_WriteByte(ETX);
  return (1); 
}


uint8_t Set_Event_Filter_REQ (SET_EVENT_FILTER_REQ_t* Request)
{
   
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(SET_EVENT_FILTER);                          // Op_code
  UART1_TX_WriteByte(1);                                         // Data_length_LSB 
  UART1_TX_WriteByte(0);                                         // Data_length_MSB
  UART1_TX_WriteByte((uint8_t)(REQ + SET_EVENT_FILTER + 1));        // Checksum
  UART1_TX_WriteByte(Request->Filter);                           // Data
  UART1_TX_WriteByte(ETX);
  return (1); 
}



uint8_t Get_Event_Filter_REQ (void)
{ 
   
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(GET_EVENT_FILTER);                          // Op_code
  UART1_TX_WriteByte(0);                                         // Data_length_LSB 
  UART1_TX_WriteByte(0);                                         // Data_length_MSB
  UART1_TX_WriteByte((uint8_t)(REQ + GET_EVENT_FILTER));            // Checksum
  UART1_TX_WriteByte(ETX);
  return (1); 
}


uint8_t Reset_REQ (void)
{
  
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(RESET);                                     // Op_code
  UART1_TX_WriteByte(0);                                         // Data_length_LSB 
  UART1_TX_WriteByte(0);                                         // Data_length_MSB
  UART1_TX_WriteByte((uint8_t)(REQ + RESET));                       // Checksum
  UART1_TX_WriteByte(ETX);
  return (1); 
}


uint8_t Restore_Factory_Settings_REQ (void)
{
  
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(RESTORE_FACTORY_SETTINGS);                  // Op_code
  UART1_TX_WriteByte(0);                                         // Data_length_LSB 
  UART1_TX_WriteByte(0);                                         // Data_length_MSB
  UART1_TX_WriteByte((uint8_t)(REQ + RESTORE_FACTORY_SETTINGS));    // Checksum
  UART1_TX_WriteByte(ETX);
  return (1); 
}


uint8_t Change_NVS_UART_Speed_REQ (CHANGE_NVS_UART_SPEED_REQ_t* Request)
{ 
   
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(CHANGE_NVS_UART_SPEED);                     // Op_code
  UART1_TX_WriteByte(1);                                         // Data_length_LSB 
  UART1_TX_WriteByte(0);                                         // Data_length_MSB
  UART1_TX_WriteByte((uint8_t)(REQ + CHANGE_NVS_UART_SPEED + 1));   // Checksum
  UART1_TX_WriteByte(Request->UartSpeed);                        // Data
  UART1_TX_WriteByte(ETX);
  return (1);   
}



uint8_t Change_UART_Settings_REQ (CHANGE_UART_SETTINGS_REQ_t* Request)
{
   
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(CHANGE_UART_SETTINGS);                      // Op_code
  UART1_TX_WriteByte(2);                                         // Data_length_LSB 
  UART1_TX_WriteByte(0);                                         // Data_length_MSB
  UART1_TX_WriteByte((uint8_t)(REQ + CHANGE_UART_SETTINGS + 2));    // Checksum
  UART1_TX_WriteByte(Request->ParityBit);                       // Data
  UART1_TX_WriteByte(Request->StopBits);                         // Data
  UART1_TX_WriteByte(ETX);
  return (1);  
}


uint8_t Test_Mode_REQ (TEST_MODE_REQ_t* Request)
{
   
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(TEST_MODE);                                 // Op_code
  UART1_TX_WriteByte(1);                                         // Data_length_LSB 
  UART1_TX_WriteByte(0);                                         // Data_length_MSB
  UART1_TX_WriteByte((uint8_t)(REQ + TEST_MODE + 1));               // Checksum
  UART1_TX_WriteByte(Request->Mode);                             // Data
  UART1_TX_WriteByte(ETX);
  return (1);  
}


uint8_t RF_Test_Mode_REQ (RF_TEST_MODE_REQ_t* Request)
{
uint8_t i;
   
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(RF_TEST_MODE);                              // Op_code
  UART1_TX_WriteByte(13);                                        // Data_length_LSB 
  UART1_TX_WriteByte(0);                                         // Data_length_MSB
  UART1_TX_WriteByte((uint8_t)(REQ + RF_TEST_MODE + 13));           // Checksum
  UART1_TX_WriteByte(Request->Test);                             // Data
  UART1_TX_WriteByte(Request->Channel);                          // Data
  UART1_TX_WriteByte(Request->PaCtrl);                           // Data
  UART1_TX_WriteByte(Request->ModulationCtrl);                   // Data
  
  for (i=0;i<8;i++)
    UART1_TX_WriteByte(Request->ModulationPattern[i]);   
  
  UART1_TX_WriteByte(Request->ModulationPayload);                // Data
  UART1_TX_WriteByte(ETX);
  return (1);  
}


uint8_t Read_RSSI_REQ (READ_RSSI_REQ_t* Request)
{ 
   
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(READ_RSSI);                                 // Op_code
  UART1_TX_WriteByte(6);                                         // Data_length_LSB 
  UART1_TX_WriteByte(0);                                         // Data_length_MSB
  UART1_TX_WriteByte((uint8_t)(REQ + READ_RSSI + 6));               // Checksum
  UART1_TX_WriteByte(Request->BdAddr.Upper_Part_MSB);            // Data
  UART1_TX_WriteByte(Request->BdAddr.Upper_Part_LSB);            // Data
  UART1_TX_WriteByte(Request->BdAddr.Middle_Part_MSB);           // Data
  UART1_TX_WriteByte(Request->BdAddr.Middle_Part_LSB);           // Data
  UART1_TX_WriteByte(Request->BdAddr.Lower_Part_MSB);            // Data
  UART1_TX_WriteByte(Request->BdAddr.Lower_Part_LSB);            // Data
  UART1_TX_WriteByte(ETX);
  return (1);  
}


uint8_t Firmware_Upgrade_REQ (void)
{ 
   
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(FIRMWARE_UPGRADE);                          // Op_code
  UART1_TX_WriteByte(0);                                         // Data_length_LSB 
  UART1_TX_WriteByte(0);                                         // Data_length_MSB
  UART1_TX_WriteByte((uint8_t)(REQ + FIRMWARE_UPGRADE));            // Checksum
  UART1_TX_WriteByte(ETX);
  return (1); 
}



