/*****************************************************************************
 *****************************************************************************
 *     _    __  __ ____  _____ ____             _          _               
 *    / \  |  \/  | __ )| ____|  _ \  __      _(_)_ __ ___| | ___  ___ ___ 
 *   / _ \ | |\/| |  _ \|  _| | |_) | \ \ /\ / / | '__/ _ \ |/ _ \/ __/ __|
 *  / ___ \| |  | | |_) | |___|  _ <   \ V  V /| | | |  __/ |  __/\__ \__ \
 * /_/   \_\_|  |_|____/|_____|_| \_\   \_/\_/ |_|_|  \___|_|\___||___/___/
 *
 *																			 
 * Module Name:																  
 * ============
 *
 *			LMX_Commands.h
 *
 * Purpose:			
 * ========
 *
 *			Header File for LMX_Commands.c 
 *
 * Author(s) & Creation Date:	
 * ==========================
 *
 *			Timo Koch, 2004/09/18
 *
 *
 * Copyright:	
 * ==========
 *
 *			AMBER wireless GmbH, Inc.
 *			Albin Koebis Strasse 18
 *			D-50858 Koeln, Germany
 *			Phone:		+49 (0) 2203-369472
 *			Fax:		+49 (0) 2203-459883
 *			Internet:	www.amber-wireless.de
 *
 *----------------------------------------------------------------
 * All software and related documentation is provided "AS IS" and 
 * without warranty or support of any kind and AMBER wireless 
 * expressly disclaims all other warranties, express or implied, 
 * including, but not limited to, the implied warranties of 
 * merchantability and fitness for a particular purpose. Under no 
 * circumstances shall AMBER wireless be liable for any 
 * incidental, special or consequential damages that result from 
 * the use or inability to use the software or related 
 * documentation, even if AMBER wireless has been advised of 
 * the liability.
 *
 * Unless otherwise stated, software written and copyrighted by 
 * AMBER wireless is distributed as "freeware". You may use 
 * and modify this software without any charge or restriction. 
 * You may distribute to others, as long as the original author 
 * is acknowledged.
 *
 * You may not use the Program in non-AMBER devices.
 *
 *****************************************************************************
 *****************************************************************************/



#ifndef _LMX_COMMANDS_H_
#define _LMX_COMMANDS_H_

#include "MY_std.h"
#include "LMX_Std.h"
#include "main.h" 

/*****************************************************************************
 * Macros & Typedefs							     *
 *****************************************************************************/

/********************************
 * Constant macros		*
 ********************************/

#define CLASS_OF_DEVICE                 0x020300  // See bluetooth.org (assigned numbers)

#define NAME_LENGTH_SPP                 15        // Max 15 bytes
#define NAME_LENGTH_DUN                 15        // Max 15 bytes
#define NAME_LENGTH_FAX                 255       // Max 255 bytes
#define NAME_LENGTH_OPP                 255       // Max 255 bytes
#define NAME_LENGTH_FTP                 255       // Max 255 bytes
#define NAME_LENGTH_SYNC                255       // Max 255 bytes 

#define RFCOMM_PORT_1     0x00000001
#define RFCOMM_PORT_2     0x00000002
#define RFCOMM_PORT_3     0x00000004
#define RFCOMM_PORT_4     0x00000008
#define RFCOMM_PORT_5     0x00000010
#define RFCOMM_PORT_6     0x00000020
#define RFCOMM_PORT_7     0x00000040
#define RFCOMM_PORT_8     0x00000080

#define RFCOMM_PORT_9     0x00000100
#define RFCOMM_PORT_10    0x00000200
#define RFCOMM_PORT_11    0x00000400
#define RFCOMM_PORT_12    0x00000800
#define RFCOMM_PORT_13    0x00001000
#define RFCOMM_PORT_14    0x00002000
#define RFCOMM_PORT_15    0x00004000
#define RFCOMM_PORT_16    0x00008000

#define RFCOMM_PORT_17    0x00010000
#define RFCOMM_PORT_18    0x00020000
#define RFCOMM_PORT_19    0x00040000
#define RFCOMM_PORT_20    0x00080000
#define RFCOMM_PORT_21    0x00100000
#define RFCOMM_PORT_22    0x00200000
#define RFCOMM_PORT_23    0x00400000
#define RFCOMM_PORT_24    0x00800000

#define RFCOMM_PORT_25    0x01000000
#define RFCOMM_PORT_26    0x02000000
#define RFCOMM_PORT_27    0x04000000
#define RFCOMM_PORT_28    0x08000000
#define RFCOMM_PORT_29    0x10000000
#define RFCOMM_PORT_30    0x20000000
#define RFCOMM_PORT_31    0x40000000
#define RFCOMM_PORT_32    0x80000000

// Opcodes
#define CHANGE_LOCAL_BDADDRESS          0x27
#define CHANGE_NVS_UART_SPEED           0x23
#define CHANGE_UART_SETTINGS            0x48
#define SET_PORTS_TO_OPEN               0x22
#define GET_PORTS_TO_OPEN               0x1F
#define RESTORE_FACTORY_SETTINGS        0x1A
#define STORE_CLASS_OF_DEVICE           0x28
#define FORCE_MASTER_ROLE               0x1D
#define READ_OPERATION_MODE             0x49
#define WRITE_OPERATION_MODE            0x4A
#define SET_DEFAULT_LINK_POLICY         0x4C
#define GET_DEFAULT_LINK_POLICY         0x4D
#define SET_EVENT_FILTER                0x4E
#define GET_EVENT_FILTER                0x4F
#define ENABLE_SDP_RECORD               0x29
#define DELETE_SDP_RECORDS              0x2A
#define STORE_SPP_RECORD                0x2B
#define STORE_DUN_RECORD                0x2C
#define STORE_FAX_RECORD                0x2D
#define STORE_OPP_RECORD                0x2E
#define STORE_FTP_RECORD                0x2F
#define STORE_SYNC_RECORD               0x30
#define RESET                           0x26
#define LMX9820_READY                   0x25
#define TEST_MODE                       0x24
#define FIRMWARE_UPGRADE                0x47
#define READ_RSSI                       0x20
#define RF_TEST_MODE                    0x4B
#define DISABLE_TL                      0x52
#define TL_ENABLED                      0x53

//The UART speed to be stored in NVS
#define UART_2400Bd    0x00 
#define UART_4800Bd    0x01
#define UART_7200Bd    0x02
#define UART_9600Bd    0x03 
#define UART_19200Bd   0x04
#define UART_38400Bd   0x05
#define UART_57600Bd   0x06
#define UART_115200Bd  0x07
#define UART_230400Bd  0x08
#define UART_460800Bd  0x09
#define UART_921600Bd  0x0A

// Msg identification ( OP-Code + Packet type)
// easier and safer handle of incoming messages
/*
Packet type identification
#define REQ     0x52
#define CFM     0x43
#define IND     0x69
#define RES     0x72
*/

#define CHANGE_LOCAL_BDADDRESS_REQ      0x2752
#define CHANGE_LOCAL_BDADDRESS_CFM      0x2743
#define CHANGE_NVS_UART_SPEED_REQ       0x2352
#define CHANGE_NVS_UART_SPEED_CFM       0x2343
#define CHANGE_UART_SETTINGS_REQ        0x4852
#define CHANGE_UART_SETTINGS_CFM        0x4843
#define SET_PORTS_TO_OPEN_REQ           0x2252
#define SET_PORTS_TO_OPEN_CFM           0x2243
#define GET_PORTS_TO_OPEN_REQ           0x1F52
#define GET_PORTS_TO_OPEN_CFM           0x1F43
#define RESTORE_FACTORY_SETTINGS_REQ    0x1A52
#define RESTORE_FACTORY_SETTINGS_CFM    0x1A43
#define STORE_CLASS_OF_DEVICE_REQ       0x2852
#define STORE_CLASS_OF_DEVICE_CFM       0x2843
#define FORCE_MASTER_ROLE_REQ           0x1D52
#define FORCE_MASTER_ROLE_CFM           0x1D43
#define READ_OPERATION_MODE_REQ         0x4952
#define READ_OPERATION_MODE_CFM         0x4943
#define WRITE_OPERATION_MODE_REQ        0x4A52
#define WRITE_OPERATION_MODE_CFM        0x4A43
#define SET_DEFAULT_LINK_POLICY_REQ     0x4C52
#define SET_DEFAULT_LINK_POLICY_CFM     0x4C43
#define GET_DEFAULT_LINK_POLICY_REQ     0x4D52
#define GET_DEFAULT_LINK_POLICY_CFM     0x4D43
#define SET_EVENT_FILTER_REQ            0x4E52
#define SET_EVENT_FILTER_CFM            0x4E43
#define GET_EVENT_FILTER_REQ            0x4F52
#define GET_EVENT_FILTER_CFM            0x4F43
#define ENABLE_SDP_RECORD_REQ           0x2952
#define ENABLE_SDP_RECORD_CFM           0x2943
#define DELETE_SDP_RECORDS_REQ          0x2A52
#define DELETE_SDP_RECORDS_CFM          0x2A43
#define STORE_SPP_RECORD_REQ            0x2B52
#define STORE_SPP_RECORD_CFM            0x2B43
#define STORE_DUN_RECORD_REQ            0x2C52
#define STORE_DUN_RECORD_CFM            0x2C43
#define STORE_FAX_RECORD_REQ            0x2D52
#define STORE_FAX_RECORD_CFM            0x2D43
#define STORE_OPP_RECORD_REQ            0x2E52
#define STORE_OPP_RECORD_CFM            0x2E43
#define STORE_FTP_RECORD_REQ            0x2F52
#define STORE_FTP_RECORD_CFM            0x2F43
#define STORE_SYNC_RECORD_REQ           0x3052
#define STORE_SYNC_RECORD_CFM           0x3043
#define RESET_REQ                       0x2652
#define DEVICE_READY_IND                0x2569
#define TEST_MODE_REQ                   0x2452
#define TEST_MODE_CFM                   0x2443
#define FIRMWARE_UPGRADE_REQ            0x4752
#define READ_RSSI_REQ                   0x2052
#define READ_RSSI_CFM                   0x2043
#define RF_TEST_MODE_REQ                0x4B52
#define RF_TEST_MODE_CFM                0x4B43
#define DISABLE_TL_REQ                  0x5252
#define DISABLE_TL_CFM                  0x5243
#define TL_ENABLED_IND                  0x5369

/********************************
 * Function macros		*
 ********************************/

/********************************
 * Typedefs			*
 ********************************/

/********************************
 * Set Default Link Policy  *
 ********************************/
typedef struct
{
  uint16_t LinkPolicy;
}SET_DEFAULT_LINK_POLICY_REQ_t;

typedef struct
{ 
  uint8_t Status;
}SET_DEFAULT_LINK_POLICY_CFM_t;


/********************************
 * Get Default Link Policy  *
 ********************************/
typedef struct
{
  uint16_t LinkPolicy;
}GET_DEFAULT_LINK_POLICY_CFM_t;


/********************************
 * Set Ports to open            *
 ********************************/
typedef struct
{
  uint32_t Ports;
}SET_PORTS_TO_OPEN_REQ_t;

typedef struct
{
  uint8_t Status;
}SET_PORTS_TO_OPEN_CFM_t;


/********************************
 * Enable SDP Record            *
 ********************************/
typedef struct
{
uint8_t State;
uint8_t Identifier;
}ENABLE_SDP_RECORD_REQ_t;

typedef struct
{
  uint8_t Status;
  uint8_t Identifier;
}ENABLE_SDP_RECORD_CFM_t;


/********************************
 * Delete SDP Record            *
 ********************************/
typedef struct
{
  uint8_t Status;
}DELETE_SDP_RECORDS_CFM_t;


/********************************
 * Store SPP Record             *
 ********************************/
typedef struct
{
  uint8_t LocalPort;
  uint8_t Authentication;
  uint8_t Encryption;
  uint8_t NameLength;
  uint8_t ServiceName[NAME_LENGTH_SPP];
}STORE_SPP_RECORD_REQ_t;

typedef struct
{
  uint8_t Status;
  uint8_t Identifier;
}STORE_SPP_RECORD_CFM_t;


/********************************
 * Store DUN Record             *
 ********************************/
typedef struct
{
  uint8_t LocalPort;
  uint8_t Authentication;
  uint8_t Encryption;
  uint8_t NameLength;
  uint8_t ServiceName[NAME_LENGTH_DUN];
}STORE_DUN_RECORD_REQ_t;

typedef struct
{
  uint8_t Status;
  uint8_t Identifier;
}STORE_DUN_RECORD_CFM_t;


/********************************
 * Store FAX Record             *
 ********************************/
typedef struct
{
  uint8_t LocalPort;
  uint8_t FaxClass1;
  uint8_t FaxClass20;
  uint8_t FaxClass2;
  uint8_t Authentication;
  uint8_t Encryption;
  uint8_t NameLength;
  uint8_t ServiceName[NAME_LENGTH_FAX];
}STORE_FAX_RECORD_REQ_t;

typedef struct
{
  uint8_t Status;
  uint8_t Identifier;
}STORE_FAX_RECORD_CFM_t;


/********************************
 * Store OPP Record             *
 ********************************/
typedef struct
{
  uint8_t LocalPort;
  uint8_t Authentication;
  uint8_t Encryption;
  uint8_t NoOfFormats;
  uint8_t SupportedFormats[NO_OF_FORMATS];
  uint8_t NameLength;
  uint8_t ServiceName[NAME_LENGTH_OPP];
}STORE_OPP_RECORD_REQ_t;

typedef struct
{
  uint8_t Status;
  uint8_t Identifier;
}STORE_OPP_RECORD_CFM_t;


/********************************
 * Store FTP Record             *
 ********************************/
typedef struct
{
  uint8_t LocalPort;
  uint8_t Authentication;
  uint8_t Encryption;
  uint8_t NameLength;
  uint8_t ServiceName[NAME_LENGTH_FTP];
}STORE_FTP_RECORD_REQ_t;

typedef struct
{
  uint8_t Status;
  uint8_t Identifier;
}STORE_FTP_RECORD_CFM_t;


/********************************
 * Store SYNC Record            *
 ********************************/
typedef struct
{
  uint8_t LocalPort;
  uint8_t Authentication;
  uint8_t Encryption;
  uint8_t NoOfDataStores;
  uint8_t SupportedDataStores[NO_OF_DATA_STORES];
  uint8_t NameLength;
  uint8_t ServiceName[NAME_LENGTH_SYNC];
}STORE_SYNC_RECORD_REQ_t;

typedef struct
{
  uint8_t Status;
  uint8_t Identifier;
}STORE_SYNC_RECORD_CFM_t;


/********************************
 * Change Local BdAddr          *
 ********************************/
typedef struct
{
  BD_ADDR_t BdAddr;
}CHANGE_LOCAL_BDADDRESS_REQ_t;

typedef struct
{
  uint8_t Status;
}CHANGE_LOCAL_BDADDRESS_CFM_t;


/********************************
 * Store Class of Device        *
 ********************************/
typedef struct
{
  uint32_t ClassOfDevice;
}STORE_CLASS_OF_DEVICE_REQ_t;

typedef struct
{
  uint8_t Status;
}STORE_CLASS_OF_DEVICE_CFM_t;


/********************************
 * Force Master Role            *
 ********************************/
typedef struct
{
uint8_t Role;
}FORCE_MASTER_ROLE_REQ_t;

typedef struct
{
  uint8_t Status;
}FORCE_MASTER_ROLE_CFM_t;


/********************************
 * Write operation mode         *
 ********************************/
typedef struct
{
  uint8_t Mode;
}WRITE_OPERATION_MODE_REQ_t;

typedef struct
{
  uint8_t Status;
}WRITE_OPERATION_MODE_CFM_t;


/********************************
 * Read operation mode          *
 ********************************/
typedef struct
{
  uint8_t Status;
  uint8_t Mode;
}READ_OPERATION_MODE_CFM_t;


/********************************
 * Set Event Filter             *
 ********************************/
typedef struct 
{
  uint8_t Filter;
}SET_EVENT_FILTER_REQ_t;

typedef struct
{
  uint8_t Status;
}SET_EVENT_FILTER_CFM_t;


/********************************
 * Get Event Filter             *
 ********************************/
typedef struct 
{
  uint8_t Filter;
}GET_EVENT_FILTER_CFM_t;


/********************************
 * Change NVS UART Speed        *
 ********************************/
typedef struct
{
  uint8_t UartSpeed;
}CHANGE_NVS_UART_SPEED_REQ_t;

typedef struct
{
  uint8_t Status;
}CHANGE_NVS_UART_SPEED_CFM_t;


/********************************
 * Change NVS UART Settings     *
 ********************************/
typedef struct 
{
  uint8_t ParityBit;
  uint8_t StopBits;
}CHANGE_UART_SETTINGS_REQ_t;

typedef struct
{
  uint8_t Status;
}CHANGE_UART_SETTINGS_CFM_t;


/********************************
 * Test Mode                    *
 ********************************/
typedef struct
{
  uint8_t Mode;
}TEST_MODE_REQ_t;

typedef struct
{
  uint8_t Status;
}TEST_MODE_CFM_t;


/********************************
 * RF Test Mode                 *
 ********************************/
typedef struct
{
  uint8_t Test;
  uint8_t Channel;
  uint8_t PaCtrl;
  uint8_t ModulationCtrl;
  uint8_t ModulationPattern[8];
  uint8_t ModulationPayload;
}RF_TEST_MODE_REQ_t;

typedef struct
{
  uint8_t Status;
}RF_TEST_MODE_CFM_t;


/********************************
 * Read RSSI                    *
 ********************************/
typedef struct
{
  BD_ADDR_t BdAddr;
}READ_RSSI_REQ_t;

typedef struct
{
  uint8_t Status;
  uint8_t RSSI;
}READ_RSSI_CFM_t;


/********************************
 * Restore Factory Settings     *
 ********************************/
typedef struct
{
  uint8_t Status;
}RESTORE_FACTORY_SETTINGS_CFM_t;


/*****************************************************************************
 * Exported Variables & Functions					     *
 *****************************************************************************/

/********************************
 * Exported Variables		*
 ********************************/

/********************************
 * Exported Functions		*
 ********************************/

extern uint8_t Set_Default_Link_Policy_REQ  (SET_DEFAULT_LINK_POLICY_REQ_t*);
extern uint8_t Get_Default_Link_Policy_REQ  (void);
extern uint8_t Disable_TL_REQ               (void);
extern uint8_t Set_Ports_to_open_REQ        (SET_PORTS_TO_OPEN_REQ_t*);
extern uint8_t Get_Ports_to_open_REQ        (void);
extern uint8_t Enable_SDP_Record_REQ        (ENABLE_SDP_RECORD_REQ_t*);
extern uint8_t Delete_SDP_Records_REQ       (void);
extern uint8_t Store_SPP_Record_REQ         (STORE_SPP_RECORD_REQ_t*);
extern uint8_t Store_DUN_Record_REQ         (STORE_DUN_RECORD_REQ_t*);
extern uint8_t Store_FAX_Record_REQ         (STORE_FAX_RECORD_REQ_t*);
extern uint8_t Store_OPP_Record_REQ         (STORE_OPP_RECORD_REQ_t*);
extern uint8_t Store_FTP_Record_REQ         (STORE_FTP_RECORD_REQ_t*);
extern uint8_t Store_SYNC_Record_REQ        (STORE_SYNC_RECORD_REQ_t*);
extern uint8_t Change_Local_BdAddr_REQ      (CHANGE_LOCAL_BDADDRESS_REQ_t*);
extern uint8_t Store_Class_of_Device_REQ    (STORE_CLASS_OF_DEVICE_REQ_t*);
extern uint8_t Force_Master_Role_REQ        (FORCE_MASTER_ROLE_REQ_t*);
extern uint8_t Read_Operation_Mode_REQ      (void);
extern uint8_t Write_Operation_Mode_REQ     (WRITE_OPERATION_MODE_REQ_t*);
extern uint8_t Set_Event_Filter_REQ         (SET_EVENT_FILTER_REQ_t*);
extern uint8_t Get_Event_Filter_REQ         (void);
extern uint8_t Reset_REQ                    (void);
extern uint8_t Restore_Factory_Settings_REQ (void);
extern uint8_t Change_NVS_UART_Speed_REQ    (CHANGE_NVS_UART_SPEED_REQ_t*);
extern uint8_t Change_UART_Settings_REQ     (CHANGE_UART_SETTINGS_REQ_t*);
extern uint8_t Test_Mode_REQ                (TEST_MODE_REQ_t*);
extern uint8_t RF_Test_Mode_REQ             (RF_TEST_MODE_REQ_t*);
extern uint8_t Read_RSSI_REQ                (READ_RSSI_REQ_t*);
extern uint8_t Firmware_Upgrade_REQ         (void);


#endif /* #ifdef _LMX_COMMANDS_H_ */

