/*****************************************************************************
 *****************************************************************************
 *     _    __  __ ____  _____ ____             _          _               
 *    / \  |  \/  | __ )| ____|  _ \  __      _(_)_ __ ___| | ___  ___ ___ 
 *   / _ \ | |\/| |  _ \|  _| | |_) | \ \ /\ / / | '__/ _ \ |/ _ \/ __/ __|
 *  / ___ \| |  | | |_) | |___|  _ <   \ V  V /| | | |  __/ |  __/\__ \__ \
 * /_/   \_\_|  |_|____/|_____|_| \_\   \_/\_/ |_|_|  \___|_|\___||___/___/
 *
 *
 * Module Name:
 * ============
 *
 *			SDAP_Commands.C
 *
 * Purpose:			
 * ========
 *
 *			Provides Functions for SDAP Commands 
 *
 * Author(s) & Creation Date:	
 * ==========================
 *
 *			Timo Koch, 2004/09/18
 *
 *
 * Copyright:	
 * ==========
 *
 *			AMBER wireless GmbH, Inc.
 *			Albin Koebis Strasse 18
 *			D-50858 Koeln, Germany
 *			Phone:		+49 (0) 2203-369472
 *			Fax:		+49 (0) 2203-459883
 *			Internet:	www.amber-wireless.de	
 *
 *----------------------------------------------------------------
 * All software and related documentation is provided "AS IS" and 
 * without warranty or support of any kind and AMBER wireless 
 * expressly disclaims all other warranties, express or implied, 
 * including, but not limited to, the implied warranties of 
 * merchantability and fitness for a particular purpose. Under no 
 * circumstances shall AMBER wireless be liable for any 
 * incidental, special or consequential damages that result from 
 * the use or inability to use the software or related 
 * documentation, even if AMBER wireless has been advised of 
 * the liability.
 *
 * Unless otherwise stated, software written and copyrighted by 
 * AMBER wireless is distributed as "freeware". You may use 
 * and modify this software without any charge or restriction. 
 * You may distribute to others, as long as the original author 
 * is acknowledged.
 *
 * You may not use the Program in non-AMBER devices.
 *
 *****************************************************************************
 *****************************************************************************/

#include "SDAP_Commands.h"

/*****************************************************************************
 * Macros & Typedefs							     *
 *****************************************************************************/

/********************************
 * Constant macros		*
 ********************************/

/********************************
 * Function macros		*
 ********************************/

/********************************
 * Typedefs			*
 ********************************/

/*****************************************************************************
 * Static / Non-static global declarations				     *
 *****************************************************************************/

/********************************
 * Non-static globals 		*
 ********************************/

/********************************
 * Static globals		*
 ********************************/


/********************************
 * Function declarations	*
 ********************************/

uint8_t SDAP_Connect_REQ             (SDAP_CONNECT_REQ_t* SDAP_Request);
uint8_t SDAP_Disconnect_REQ          (void);
uint8_t SDAP_Service_Browse_REQ      (SDAP_SERVICE_BROWSE_REQ_t* SDAP_Request);
uint8_t SDAP_Service_Search_REQ      (SDAP_SERVICE_SEARCH_REQ_t* SDAP_Request);
uint8_t SDAP_Service_Request_REQ     (SDAP_SERVICE_REQUEST_REQ_t* SDAP_Request);
uint8_t SDAP_Attribute_Request_REQ   (SDAP_ATTRIBUTE_REQUEST_REQ_t* SDAP_Request);

/*****************************************************************************
 * Function definitions							     * 
 *****************************************************************************/

uint8_t SDAP_Connect_REQ (SDAP_CONNECT_REQ_t* SDAP_Request)
{
  
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(SDAP_CONNECT);                              // Op_code
  UART1_TX_WriteByte(6);                                         // Data_length_LSB
  UART1_TX_WriteByte(0);                                         // Data_length_MSB 
  UART1_TX_WriteByte((uint8_t)(REQ + SDAP_CONNECT + 6));            // Checksum
  UART1_TX_WriteByte(SDAP_Request->BdAddr.Upper_Part_MSB);       // Data
  UART1_TX_WriteByte(SDAP_Request->BdAddr.Upper_Part_LSB);       // Data
  UART1_TX_WriteByte(SDAP_Request->BdAddr.Middle_Part_MSB);      // Data
  UART1_TX_WriteByte(SDAP_Request->BdAddr.Middle_Part_LSB);      // Data
  UART1_TX_WriteByte(SDAP_Request->BdAddr.Lower_Part_MSB);       // Data
  UART1_TX_WriteByte(SDAP_Request->BdAddr.Lower_Part_LSB);       // Data
  UART1_TX_WriteByte(ETX);                                       // End_delimiter
  
  return (0);                                                               // OK
} 

uint8_t SDAP_Disconnect_REQ (void)
{
  
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(SDAP_DISCONNECT);                           // Op_code
  UART1_TX_WriteByte(0);                                         // Data_length_MSB 
  UART1_TX_WriteByte(0);                                         // Data_length_LSB
  UART1_TX_WriteByte((uint8_t)(REQ + SDAP_DISCONNECT + 0));         // Checksum
  UART1_TX_WriteByte(ETX);                                       // End_delimiter
  
  return (0);                                                               // OK
}

uint8_t SDAP_Service_Browse_REQ (SDAP_SERVICE_BROWSE_REQ_t* SDAP_Request)
{

  
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(SDAP_SERVICE_BROWSE);                       // Op_code
  UART1_TX_WriteByte(2);                                         // Data_length_LSB
  UART1_TX_WriteByte(0);                                         // Data_length_MSB 
  UART1_TX_WriteByte((uint8_t)(REQ + SDAP_SERVICE_BROWSE + 2));     // Checksum
  UART1_TX_WriteByte(*((uint8_t*)&(SDAP_Request->BrowseGroupID)+ 0));        // Data
  UART1_TX_WriteByte(*((uint8_t*)&(SDAP_Request->BrowseGroupID)+ 1));        // Data
  UART1_TX_WriteByte(ETX);                                       // End_delimiter
  
  return (0);                                                               // OK
}

uint8_t SDAP_Service_Search_REQ (SDAP_SERVICE_SEARCH_REQ_t* SDAP_Request)
{
uint8_t i;
  
  i =  2 + 2*SDAP_Request->SearchPatternLength + 2*SDAP_Request->AttributesLength;
  
  
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(SDAP_SERVICE_SEARCH);                       // Op_code
  UART1_TX_WriteByte((uint8_t)i);                                   // Data_length_LSB
  UART1_TX_WriteByte((uint8_t)(i>>8));                              // Data_length_MSB 
  UART1_TX_WriteByte((uint8_t)(REQ + SDAP_SERVICE_SEARCH + i));     // Checksum
  UART1_TX_WriteByte(SDAP_Request->SearchPatternLength);         // Data
  
  for (i=0;i < SDAP_Request->SearchPatternLength; i++)
    {
      UART1_TX_WriteByte((uint8_t)(SDAP_Request->SearchPattern[i]>>8));// Data
      UART1_TX_WriteByte((uint8_t)(SDAP_Request->SearchPattern[i]));// Data
    }  
  UART1_TX_WriteByte(SDAP_Request->AttributesLength);            // Data
  
  for (i=0;i < SDAP_Request->AttributesLength; i++)
    {
      UART1_TX_WriteByte((uint8_t)(SDAP_Request->Attributes[i]>>8));// Data
      UART1_TX_WriteByte((uint8_t)(SDAP_Request->Attributes[i]));   // Data
    }
  UART1_TX_WriteByte(ETX);                                       // End_delimiter
  
  return (0);                                                               // OK
}



uint8_t SDAP_Service_Request_REQ (SDAP_SERVICE_REQUEST_REQ_t* SDAP_Request)
{
uint8_t i;
  i = 1 + 2*SDAP_Request->SearchPatternLength;
  
  
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(SDAP_SERVICE_REQUEST);                      // Op_code
  UART1_TX_WriteByte((uint8_t)i);                                   // Data_length_LSB
  UART1_TX_WriteByte((uint8_t)(i>>8));                              // Data_length_MSB 
  UART1_TX_WriteByte((uint8_t)(REQ + SDAP_SERVICE_REQUEST + i));    // Checksum
  UART1_TX_WriteByte(SDAP_Request->SearchPatternLength);         // Data
  
  for (i=0;i < SDAP_Request->SearchPatternLength; i++)
    {
      UART1_TX_WriteByte((uint8_t)(SDAP_Request->SearchPattern[i]>>8)); // Data
      UART1_TX_WriteByte((uint8_t)(SDAP_Request->SearchPattern[i]));// Data
    } 
  UART1_TX_WriteByte(ETX);                                       // End_delimiter
  
  return (0);                                                               // OK
}


uint8_t SDAP_Attribute_Request_REQ (SDAP_ATTRIBUTE_REQUEST_REQ_t* SDAP_Request)
{
uint8_t i;
  i = 5 + 2*SDAP_Request->AttributesLength;
  
  
  UART1_TX_WriteByte(STX);                                       // Start_delimiter
  UART1_TX_WriteByte(REQ);                                       // Packet_Type_identification
  UART1_TX_WriteByte(SDAP_ATTRIBUTE_REQUEST);                    // Op_code
  UART1_TX_WriteByte((uint8_t)i);                                   // Data_length_LSB
  UART1_TX_WriteByte((uint8_t)(i>>8));                              // Data_length_MSB 
  UART1_TX_WriteByte((uint8_t)(REQ + SDAP_ATTRIBUTE_REQUEST + i));  // Checksum
  UART1_TX_WriteByte((uint8_t)(SDAP_Request->Handle >> 24));        // Data
  UART1_TX_WriteByte((uint8_t)(SDAP_Request->Handle >> 16));        // Data
  UART1_TX_WriteByte((uint8_t)(SDAP_Request->Handle >> 8));         // Data
  UART1_TX_WriteByte((uint8_t)(SDAP_Request->Handle));              // Data
  
  UART1_TX_WriteByte(SDAP_Request->AttributesLength);            // Data
  
  for (i=0;i < SDAP_Request->AttributesLength; i++)
    {
     UART1_TX_WriteByte((uint8_t)(SDAP_Request->Attributes[i] >> 8));  // Data
     UART1_TX_WriteByte((uint8_t)(SDAP_Request->Attributes[i]));    // Data
    }
  UART1_TX_WriteByte(ETX);                                       // End_delimiter
  
  return (0);                                                               // OK
}







