/*****************************************************************************
 *****************************************************************************
 *     _    __  __ ____  _____ ____             _          _               
 *    / \  |  \/  | __ )| ____|  _ \  __      _(_)_ __ ___| | ___  ___ ___ 
 *   / _ \ | |\/| |  _ \|  _| | |_) | \ \ /\ / / | '__/ _ \ |/ _ \/ __/ __|
 *  / ___ \| |  | | |_) | |___|  _ <   \ V  V /| | | |  __/ |  __/\__ \__ \
 * /_/   \_\_|  |_|____/|_____|_| \_\   \_/\_/ |_|_|  \___|_|\___||___/___/
 *
 *																			 
 * Module Name:																  
 * ============
 *
 *			SDAP_Commands.h
 *
 * Purpose:			
 * ========
 *
 *			Header File for SDAP_Commands.c
 *
 * Author(s) & Creation Date:	
 * ==========================
 *
 *			Timo Koch, 2004/09/18
 *
 *
 * Copyright:	
 * ==========
 *
 *			AMBER wireless GmbH, Inc.
 *			Albin Koebis Strasse 18
 *			D-50858 Koeln, Germany
 *			Phone:		+49 (0) 2203-369472
 *			Fax:		+49 (0) 2203-459883
 *			Internet:	www.amber-wireless.de
 *
 *----------------------------------------------------------------
 * All software and related documentation is provided "AS IS" and 
 * without warranty or support of any kind and AMBER wireless 
 * expressly disclaims all other warranties, express or implied, 
 * including, but not limited to, the implied warranties of 
 * merchantability and fitness for a particular purpose. Under no 
 * circumstances shall AMBER wireless be liable for any 
 * incidental, special or consequential damages that result from 
 * the use or inability to use the software or related 
 * documentation, even if AMBER wireless has been advised of 
 * the liability.
 *
 * Unless otherwise stated, software written and copyrighted by 
 * AMBER wireless is distributed as "freeware". You may use 
 * and modify this software without any charge or restriction. 
 * You may distribute to others, as long as the original author 
 * is acknowledged.
 *
 * You may not use the Program in non-AMBER devices.
 *
 *****************************************************************************
 *****************************************************************************/



#ifndef _SDAP_COMMANDS_H_
#define _SDAP_COMMANDS_H_

#include "LMX_Std.h"
#include "MY_std.h"
#include "main.h"

/*****************************************************************************
 * Macros & Typedefs							     *
 *****************************************************************************/


/********************************
 * Constant macros		*
 ********************************/

#define SERVICE_NAME_LENGTH   30     

#define UUID_PublicBrowseGroup  0x1002  // Returns the list of all registered services
#define UUID_SPP                0x1101  // UUID of Serial Port Profile
#define UUID_DUN                0x1103  // UUID of Dial-Up Networking Profile
#define UUID_FAX                0x1111  // UUID of Fax Profile 
#define UUID_OPP                0x1105  // UUID of Object Push Profile
#define UUID_FTP                0x1106  // UUID of File Transfer Profile
#define UUID_SYNC               0x1104  // UUID of Synchronization Profile     

// Opcodes

#define SDAP_CONNECT                    0x32
#define SDAP_DISCONNECT                 0x33
#define SDAP_CONNECTION_LOST            0x34
#define SDAP_SERVICE_BROWSE             0x35
#define SDAP_SERVICE_SEARCH             0x36
#define SDAP_SERVICE_REQUEST            0x1E
#define SDAP_ATTRIBUTE_REQUEST          0x3F

// Msg identification ( OP-Code + Packet type)
// easier and safer handle of incoming messages
/*
Packet type identification
#define REQ     0x52
#define CFM     0x43
#define IND     0x69
#define RES     0x72
*/

#define SDAP_CONNECT_REQ                0x3252
#define SDAP_CONNECT_CFM                0x3243
#define SDAP_DISCONNECT_REQ             0x3352
#define SDAP_DISCONNECT_CFM             0x3343
#define SDAP_CONNECTION_LOST_IND        0x3469

#define SDAP_SERVICE_BROWSE_REQ         0x3552
#define SDAP_SERVICE_BROWSE_CFM         0x3543
#define SDAP_SERVICE_SEARCH_REQ         0x3652
#define SDAP_SERVICE_SEARCH_CFM         0x3643
#define SDAP_SERVICE_REQUEST_REQ        0x1E52
#define SDAP_SERVICE_REQUEST_CFM        0x1E43
#define SDAP_ATTRIBUTE_REQUEST_REQ      0x3F52
#define SDAP_ATTRIBUTE_REQUEST_CFM      0x3F43

/********************************
 * Function macros		*
 ********************************/

/********************************
 * Typedefs			*
 ********************************/

/********************************
 * SDAP Connect                 *  
 ********************************/
typedef struct
{
  BD_ADDR_t BdAddr;
}SDAP_CONNECT_REQ_t;

typedef struct
{
  uint8_t Status;
}SDAP_CONNECT_CFM_t;


/********************************
 * SDAP Disconnect              *  
 ********************************/
typedef struct
{
  uint8_t Status;
}SDAP_DISCONNECT_CFM_t;


/********************************
 * SDAP Service Search          *  
 ********************************/
typedef struct
{
  uint8_t SearchPatternLength;
  uint16_t SearchPattern[MAX_SEARCH_PATTERN];
  uint8_t AttributesLength;
  uint16_t Attributes[MAX_ATTRIBUTES];
}SDAP_SERVICE_SEARCH_REQ_t;

typedef struct
{
  uint8_t Status;
  uint16_t  Length;
  uint8_t Result[MAX_PACKET_LENGHT];
}SDAP_SERVICE_SEARCH_CFM_t;


/********************************
 * SDAP Service Browse          *  
 ********************************/
typedef struct
{
  uint16_t  BrowseGroupID;
  uint16_t  ServiceID;
  uint8_t PortNr;
  uint8_t NameLength;
  uint8_t ServiceName[SERVICE_NAME_LENGTH];
}SDAP_SERVICE_t;

typedef struct
{
 uint16_t BrowseGroupID;
}SDAP_SERVICE_BROWSE_REQ_t;

typedef struct
{
 uint8_t  Status;
 uint8_t  NoOfServices;
 SDAP_SERVICE_t Service[10];
}SDAP_SERVICE_BROWSE_CFM_t;


/********************************
 * SDAP Service Request         *  
 ********************************/
typedef struct 
{
  uint8_t SearchPatternLength;
  uint16_t SearchPattern[MAX_SEARCH_PATTERN];
}SDAP_SERVICE_REQUEST_REQ_t;

typedef struct 
{
  uint8_t Status;
  uint16_t  Length;
  uint32_t Result; 
}SDAP_SERVICE_REQUEST_CFM_t;


/********************************
 * SDAP Attribute Request       *  
 ********************************/
typedef struct
{
  uint32_t Handle;
  uint8_t AttributesLength;
  uint16_t Attributes[MAX_ATTRIBUTES];
}SDAP_ATTRIBUTE_REQUEST_REQ_t;

typedef struct
{
  uint8_t Status;
  uint16_t  Length;
  uint8_t Result[MAX_PACKET_LENGHT];
}SDAP_ATTRIBUTE_REQUEST_CFM_t;

/*****************************************************************************
 * Exported Variables & Functions					     *
 *****************************************************************************/

/********************************
 * Exported Variables		*
 ********************************/

/********************************
 * Exported Functions		*
 ********************************/

extern uint8_t SDAP_Connect_REQ             (SDAP_CONNECT_REQ_t*);
extern uint8_t SDAP_Disconnect_REQ          (void);
extern uint8_t SDAP_Service_Browse_REQ      (SDAP_SERVICE_BROWSE_REQ_t*);
extern uint8_t SDAP_Service_Search_REQ      (SDAP_SERVICE_SEARCH_REQ_t*);
extern uint8_t SDAP_Service_Request_REQ     (SDAP_SERVICE_REQUEST_REQ_t*);
extern uint8_t SDAP_Attribute_Request_REQ   (SDAP_ATTRIBUTE_REQUEST_REQ_t*);


#endif /* #ifdef _SDAP_COMMANDS_H_ */

