/*****************************************************************************
 *****************************************************************************
 *     _    __  __ ____  _____ ____             _          _               
 *    / \  |  \/  | __ )| ____|  _ \  __      _(_)_ __ ___| | ___  ___ ___ 
 *   / _ \ | |\/| |  _ \|  _| | |_) | \ \ /\ / / | '__/ _ \ |/ _ \/ __/ __|
 *  / ___ \| |  | | |_) | |___|  _ <   \ V  V /| | | |  __/ |  __/\__ \__ \
 * /_/   \_\_|  |_|____/|_____|_| \_\   \_/\_/ |_|_|  \___|_|\___||___/___/
 *
 *
 * Module Name:
 * ============
 *
 *			main.c
 *
 * Purpose:			
 * ========
 *
 *              Application Note
 *              Configuration and using transparent mode of LMX9820     			 
 *
 * Author(s) & Creation Date:	
 * ==========================
 *
 *			Timo Koch, 2004/09/18
 *
 *
 * Copyright:	
 * ==========
 *
 *			AMBER wireless GmbH, Inc.
 *			Albin Koebis Strasse 18
 *			D-50858 Koeln, Germany
 *			Phone:		+49 (0) 2203-369472
 *			Fax:		+49 (0) 2203-459883
 *			Internet:	www.amber-wireless.de	
 *
 *----------------------------------------------------------------
 * All software and related documentation is provided "AS IS" and 
 * without warranty or support of any kind and AMBER wireless 
 * expressly disclaims all other warranties, express or implied, 
 * including, but not limited to, the implied warranties of 
 * merchantability and fitness for a particular purpose. Under no 
 * circumstances shall AMBER wireless be liable for any 
 * incidental, special or consequential damages that result from 
 * the use or inability to use the software or related 
 * documentation, even if AMBER wireless has been advised of 
 * the liability.
 *
 * Unless otherwise stated, software written and copyrighted by 
 * AMBER wireless is distributed as "freeware". You may use 
 * and modify this software without any charge or restriction. 
 * You may distribute to others, as long as the original author 
 * is acknowledged.
 *
 * You may not use the Program in non-AMBER devices.
 *
 *****************************************************************************
 *****************************************************************************/


// UART 1 -> LMX9820


// Includes 
#include "main.h"

// Functions 
void     MAIN_Msg_Handling        (LMX_Command_Protocol_t* Msg);
void     MAIN_Init                (void); 
void     MAIN_Check_Protocol_LMX  (uint8_t Data_Byte);
uint16_t UART1_TX_WriteByte       (uint8_t ucDataByte);

// Variables
LMX_Command_Protocol_t* MAIN_LMX_Command_Protocol;
uint16_t                MAIN_RX_Lenght_LMX;
uint16_t                MAIN_Help_LMX;
uint8_t                 MAIN_Data_Array[50];
uint8_t                 MAIN_UART1_TX_Buffer[200];
uint8_t                 MAIN_Config_Statecounter;
uint8_t                 MAIN_Transparent_Mode;
uint8_t*                MAIN_P_UART1_TX_Write_Pointer;
uint8_t*                MAIN_P_UART1_TX_Read_Pointer;
uint16_t                MAIN_UART_TX_Buffer_Load;
uint16_t                MAIN_Config_Timer;
uint16_t                MAIN_Protocol_Timer;

enum 
{
WAIT_STX =        0,
WAIT_TYPE =       1,
WAIT_OPCODE =     2,
WAIT_LENGHT_LSB = 3,
WAIT_LENGHT_MSB = 4,
WAIT_CRC =        5,
WAIT_DATA =       6,
WAIT_ETX =        7,
}
MAIN_Check_Protocol_LMX_Statecounter;



/*****************************************************************************
 * Function definitions														 * 
 *****************************************************************************/
/* --------------------------------------------------------------------------
 *
 * MAIN_Init()
 *
 * System init      
 *  
 * 
 */
void MAIN_Init (void)
{
  uint8_t i;
  uint16_t Baud_Register;
  float_t ybin1;
  float_t ybin2;
  uint8_t Overflow = 1;
    
  // Watchdog stop
  WDTCTL = WDTPW + WDTHOLD;
  
  // Reset LMX (portdef.h)
  RST_RESET_BT();
  
  // LMX9820 UART Setting: 9600/8/N/1
  RST_ISEL_2();
  SET_ISEL_1();
  
  //Ports init (portdef.h)
  INIT_PORT_1();
  INIT_PORT_2();
  INIT_PORT_3();
  INIT_PORT_4();
  INIT_PORT_5();
  INIT_PORT_6();

  // Turn on XT2 and wait until XT2 is stable
  BCSCTL1 &= ~XT2OFF;	
  do 
  {
    SWITCH_LED_RX_OUTPUT();
    IFG1 &= ~OFIFG;				
    for (i = 0xFF; i > 0; i--);
		  
  } while ((IFG1 & OFIFG) != 0);

  //Clock init
  //MCLK -> XT2
  //SMCLK -> XT2
  BCSCTL2 = SELM1 + SELS;
  
  // Timer init
  TACTL |= (TASSEL_2 | TACLR);                              // SMCLK, clear TAR
  TACCTL0 |= CCIE;                                          // CCR0 interrupt enabled
  TACCR0 = MAIN_TIMER_PRESCALER_1MS;                        // 1ms Prescaler 
  TAR = 0;                                                  // Timer clear
  TACTL|= MC1;                                              // Start Timer_a in continous mode
  
  // UART Init  9600/8/N/1 @ 4MHz
  ME2 &= ~(UTXE1 + URXE1);
  IE2 &= ~(UTXIE1 + URXIE1);
  UTCTL1 = SSEL1;												// UCLK = SMCLK 
  U1CTL = CHAR;
  
  Baud_Register = 4000000 / 9600;
  
  UBR01 = Baud_Register & 0xFF;
  UBR11 = (Baud_Register >> 8) & 0xFF;

  ybin1 = (4000000 / 9600) - Baud_Register;
  ybin2 = ybin1;
	
  for (i=0;i<8;i++)
  {
    if((ybin1 + ybin2) > Overflow)
      {
	Overflow++;
	Baud_Register |= (0x01 << i);
      }
    ybin2 = ybin1 + ybin2;
  }

  UMCTL1 = Baud_Register;

  URCTL1 = 0x08;
    
  ME2 |= UTXE1 + URXE1;
  IE2 |= URXIE1;
  IE2 |= UTXIE1;		                                        	
  
  MAIN_P_UART1_TX_Write_Pointer = (uint8_t*)&MAIN_UART1_TX_Buffer[0];
  MAIN_P_UART1_TX_Read_Pointer = (uint8_t*)&MAIN_UART1_TX_Buffer[0];
  MAIN_Transparent_Mode = 0;
  MAIN_Config_Statecounter = 1;
  MAIN_LMX_Command_Protocol = (LMX_Command_Protocol_t*) &MAIN_Data_Array[0];
  GLOBAL_INTERRUPT_ENABLE();
  // Now turn on the LMX
  SET_RESET_BT();  
}


/* --------------------------------------------------------------------------
 *
 * main()
 *
 *      
 *  
 * 
 */
void main (void)
{
MAIN_Init(); // system init    

while(1) // loop forever
{
if((U1TCTL & TXEPT) && MAIN_UART_TX_Buffer_Load)
  IFG1 |= UTXIFG0;  // force interrupt	
}
}





/* --------------------------------------------------------------------------
 *
 * MAIN_Msg_Handling_LMX()
 *
 * Message Handler      
 *  
 * 
 */
void MAIN_Msg_Handling_LMX(LMX_Command_Protocol_t* Msg)
{
uint16_t Msg_Type;

Msg_Type = (Msg->Op_code << 8) & 0xff00;
Msg_Type += Msg->Packet_Type_identification;

  switch (Msg_Type)
  {
    case DEVICE_READY_IND:
        {
          // Now is the LMX ready
          //  Data don't care  
            switch (MAIN_Config_Statecounter)
            {
            // First Restore LMX Factory Settings 
            case 1:
                Restore_Factory_Settings_REQ();
            break;
            // Second set Device Name
            case 2:
              {
                GAP_WRITE_LOCAL_NAME_REQ_t* GAP_Request; 
                GAP_Request = (GAP_WRITE_LOCAL_NAME_REQ_t*)MAIN_Data_Array;
                                
                GAP_Request->NameLength = 12;
                
                GAP_Request->DeviceName[0]   = 'A';
                GAP_Request->DeviceName[1]   = 'M';
                GAP_Request->DeviceName[2]   = 'B';
                GAP_Request->DeviceName[3]   = 'E';
                GAP_Request->DeviceName[4]   = 'R';
                GAP_Request->DeviceName[5]   = ' ';
                GAP_Request->DeviceName[6]   = 'S';
                GAP_Request->DeviceName[7]   = 'l';
                GAP_Request->DeviceName[8]   = 'a';
                GAP_Request->DeviceName[9]   = 'v';
                GAP_Request->DeviceName[10]  = 'e';
                GAP_Request->DeviceName[11]  = 0x00;
                GAP_Write_Local_Name_REQ(GAP_Request);
              }
            break;
            // Third set Class of Device
            case 3:
              {
                  STORE_CLASS_OF_DEVICE_REQ_t* Request;
                  Request = (STORE_CLASS_OF_DEVICE_REQ_t*)MAIN_Data_Array;
                  
                  Request->ClassOfDevice = 0x023000;
                  Store_Class_of_Device_REQ(Request);
              }
            break;
            // Fourth Set Event Filter   
            case 4:
              {
                  SET_EVENT_FILTER_REQ_t* Request;
                  Request = (SET_EVENT_FILTER_REQ_t*)MAIN_Data_Array;
                  
                  Request->Filter = 0x00;
                  Set_Event_Filter_REQ(Request);
              }
            break;
            //  fifth UART Speed   
            case 5:
              {
                CHANGE_NVS_UART_SPEED_REQ_t Request;
                // 0x00 = 2400 Baud
                // 0x01 = 4800 Baud
                // 0x02 = 7200 Baud
                // 0x03 = 9600 Baud 
                // 0x04 = 19200 Baud
                // 0x05 = 38400 Baud
                // 0x06 = 57600 Baud
                // 0x07 = 115200 Baud
                // 0x08 = 230400 Baud
                // 0x09 = 460800 Baud
                // 0x0A = 921600 Baud
                Request.UartSpeed = 0x07;
                
                Change_NVS_UART_Speed_REQ(&Request);
              }
            break;
            case 6:
              {  
                GAP_SET_FIXED_PIN_REQ_t* Request;
                Request = (GAP_SET_FIXED_PIN_REQ_t*)MAIN_Data_Array;
                
                Request->Pinlength = 4;
                Request->Pincode[0] = '0';
                Request->Pincode[1] = '0';
                Request->Pincode[2] = '0';
                Request->Pincode[3] = '0';
                
                GAP_Set_Fixed_Pin_REQ (Request); 
              }
            break;
            // Config LMX9820 OK         
            case 7:
                MAIN_Config_Statecounter = 1;
            break;
            
            default:
                MAIN_Config_Statecounter = 1;
            break;
            }
          
          }    
          
     
    break;    
    
// Confirm and Indication Management 
    
    case RESTORE_FACTORY_SETTINGS_CFM:
      {
        RESTORE_FACTORY_SETTINGS_CFM_t* Data  = (RESTORE_FACTORY_SETTINGS_CFM_t*)Msg->Packet_Data;
        
        if(Data->Status == ERROR_OK)     
          { 
            
            MAIN_Config_Timer = MAIN_ONE_SECOND;
            
            while(!MAIN_Config_Timer);
            
            MAIN_Config_Statecounter = 2;
            Reset_REQ();
            
          }
          else;
            
      }
   break; 
   
   case  GAP_SET_FIXED_PIN_CFM:
      {    
          GAP_SET_FIXED_PIN_CFM_t* Data  = (GAP_SET_FIXED_PIN_CFM_t*)Msg->Packet_Data;
                    
          if(Data->Status == ERROR_OK)     
          { 
            MAIN_Config_Timer = MAIN_ONE_SECOND;
            
            while(!MAIN_Config_Timer);
            
            MAIN_Config_Statecounter = 7;
            Reset_REQ();
            
          }
          else;
      }
   break;
           
    case GAP_WRITE_LOCAL_NAME_CFM: 
        {
          GAP_WRITE_LOCAL_NAME_CFM_t* Data  = (GAP_WRITE_LOCAL_NAME_CFM_t*)Msg->Packet_Data;
                    
          if(Data->Status == ERROR_OK)     
          { 
            MAIN_Config_Timer = MAIN_ONE_SECOND;
            
            while(!MAIN_Config_Timer);
            
            MAIN_Config_Statecounter = 3;
            Reset_REQ();
            
          }
          else;
            
        }  
    break;
   
   case STORE_CLASS_OF_DEVICE_CFM:
      {
        STORE_CLASS_OF_DEVICE_CFM_t* Data  = (STORE_CLASS_OF_DEVICE_CFM_t*)Msg->Packet_Data;
                    
          if(Data->Status == ERROR_OK)     
          { 
                     
            MAIN_Config_Timer = MAIN_ONE_SECOND;
            
            while(!MAIN_Config_Timer);
            
            MAIN_Config_Statecounter = 4;
            Reset_REQ();
            
          }
          else;
            
      }
   break;        
   
   case SET_EVENT_FILTER_CFM:
      {
        SET_EVENT_FILTER_CFM_t* Data  = (SET_EVENT_FILTER_CFM_t*)Msg->Packet_Data;
                    
          if(Data->Status == ERROR_OK)     
          { 
            MAIN_Config_Timer = MAIN_ONE_SECOND;
            
            while(!MAIN_Config_Timer);
            
            MAIN_Config_Statecounter = 5;
            Reset_REQ();
            
          }
          else;
            
      }
   break;        
   
   case CHANGE_NVS_UART_SPEED_CFM:
      {
        CHANGE_NVS_UART_SPEED_CFM_t* Data  = (CHANGE_NVS_UART_SPEED_CFM_t*)Msg->Packet_Data;
                    
          if(Data->Status == ERROR_OK)     
          { 
                        
            MAIN_Config_Timer = MAIN_ONE_SECOND;
            
            while(!MAIN_Config_Timer);
            
            MAIN_Config_Statecounter = 6;
            Reset_REQ();
            
          }
          else;
            
      }
   break;        
   
      
   case SPP_INCOMMING_LINK_ESTABLISHED_IND:
       // Now connected and in Transparent Mode!!
       MAIN_Transparent_Mode = 1;   
   break;
   
   case SPP_TRANSPARENT_MODE_IND:
       MAIN_Transparent_Mode = 0;   
   break;
   
   case SPP_LINK_RELEASED_IND:
      // your application - in this case break-signal is used for disconnect
   break; 
   default:
      // all other messages don't care 
   break;
  }  
}







/* --------------------------------------------------------------------------
 *
 * MAIN_Check_Protocol_LMX()
 *
 * LMX UART Protocol       
 *  
 * 
 */
void MAIN_Check_Protocol_LMX(uint8_t Data_Byte)
{

// Transparent Mode ? 
if(MAIN_Transparent_Mode)
  return; // your application   
  
// Check incoming Message  
switch (MAIN_Check_Protocol_LMX_Statecounter)  
  {
  case WAIT_STX:    
      if (Data_Byte == STX) 
        {
          MAIN_Check_Protocol_LMX_Statecounter = WAIT_TYPE;
          MAIN_Protocol_Timer = MAIN_TIMEOUT_LMX;
        }  
  break;
  
  case WAIT_TYPE:
      if(Data_Byte == REQ || Data_Byte == CFM || Data_Byte == IND || Data_Byte == RES)
        {
          MAIN_LMX_Command_Protocol->Packet_Type_identification = Data_Byte; 
          MAIN_Check_Protocol_LMX_Statecounter = WAIT_OPCODE;
          MAIN_LMX_Command_Protocol->Checksum = MAIN_LMX_Command_Protocol->Packet_Type_identification;
          MAIN_Protocol_Timer = MAIN_TIMEOUT_LMX;
        }
       else
          MAIN_Check_Protocol_LMX_Statecounter = WAIT_STX;
  break;
  
  case WAIT_OPCODE:
      MAIN_LMX_Command_Protocol->Op_code = Data_Byte; 
      MAIN_LMX_Command_Protocol->Checksum += MAIN_LMX_Command_Protocol->Op_code;
      MAIN_Protocol_Timer = MAIN_TIMEOUT_LMX;
      MAIN_Check_Protocol_LMX_Statecounter = WAIT_LENGHT_LSB;
  break;
      
  case WAIT_LENGHT_LSB:
      MAIN_LMX_Command_Protocol->Data_length_LSB = Data_Byte; 
      MAIN_Protocol_Timer = MAIN_TIMEOUT_LMX;
      MAIN_Check_Protocol_LMX_Statecounter = WAIT_LENGHT_MSB;
  break;
  
  case WAIT_LENGHT_MSB:
      MAIN_LMX_Command_Protocol->Data_length_MSB = Data_Byte; 
      // Calc Length Byte 
      MAIN_RX_Lenght_LMX = MAIN_LMX_Command_Protocol->Data_length_MSB;      
      MAIN_RX_Lenght_LMX = MAIN_RX_Lenght_LMX << 8;
      MAIN_RX_Lenght_LMX += MAIN_LMX_Command_Protocol->Data_length_LSB;
            
      MAIN_LMX_Command_Protocol->Checksum += MAIN_RX_Lenght_LMX;
      MAIN_Protocol_Timer = MAIN_TIMEOUT_LMX;
      MAIN_Check_Protocol_LMX_Statecounter = WAIT_CRC;
      if(MAIN_RX_Lenght_LMX > MAX_PACKET_LENGHT)
        MAIN_Check_Protocol_LMX_Statecounter = WAIT_STX;
      
  break;
  
  case WAIT_CRC:
      if(MAIN_LMX_Command_Protocol->Checksum == Data_Byte)
        {
          MAIN_Protocol_Timer = MAIN_TIMEOUT_LMX;
          MAIN_Check_Protocol_LMX_Statecounter = WAIT_DATA;
          MAIN_Help_LMX = 0;  
          if (!MAIN_RX_Lenght_LMX)
            MAIN_Check_Protocol_LMX_Statecounter = WAIT_ETX;
        }
      else
          MAIN_Check_Protocol_LMX_Statecounter = WAIT_STX;
        
  break;
  
  case WAIT_DATA:
       MAIN_LMX_Command_Protocol->Packet_Data[MAIN_Help_LMX] = Data_Byte;
       MAIN_Help_LMX++;
       MAIN_RX_Lenght_LMX--;
        
      if (!MAIN_RX_Lenght_LMX)
        MAIN_Check_Protocol_LMX_Statecounter = WAIT_ETX;
       
       MAIN_Protocol_Timer = MAIN_TIMEOUT_LMX;
  break;
     
  case WAIT_ETX: 
      if (Data_Byte == ETX);
        {
          MAIN_Msg_Handling_LMX(MAIN_LMX_Command_Protocol);
        }  
      MAIN_Check_Protocol_LMX_Statecounter = WAIT_STX;
  break;
  
  default:
      MAIN_Check_Protocol_LMX_Statecounter = WAIT_STX;
  break;
  }
}






/* IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
 *
 * UART1_RX_Interrupt()
 *
 *
 *
 *
 */
#if __VER__ >= 300   
  #pragma vector = UART1RX_VECTOR 
  __interrupt void UART1_RX_Interrupt(void)
#else
  interrupt [UART1RX_VECTOR] void UART1_RX_Interrupt (void)
#endif  
{
if (URCTL1 & 0x1)                  // error Flags 
  {
    uint8_t UART_dummy;
    
    if (URCTL1 & 0x10)            // Break
      MAIN_Transparent_Mode = 0;  // Leave Transparent Mode     
    
    UART_dummy = U1RXBUF;         // Flush RX buffer 
    UART_dummy = UART_dummy >> 1; // and avoid compiler error
  }
else
  { 
    if(!MAIN_Protocol_Timer)
        MAIN_Check_Protocol_LMX_Statecounter = WAIT_STX;
  
    MAIN_Check_Protocol_LMX(U1RXBUF);
  }
}/* UART_RX_Interrupt() */




/* IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
 *
 * UART1_TX_Interrupt()
 *
 *
 *
 *
 */
#if __VER__ >= 300   
  #pragma vector = UART1TX_VECTOR 
  __interrupt void UART1_TX_Interrupt(void)
#else
 interrupt [UART1TX_VECTOR] void UART1_TX_Interrupt (void)
#endif  
{     
      if(!MAIN_UART_TX_Buffer_Load)
        return;
        
      U1TXBUF = *MAIN_P_UART1_TX_Read_Pointer;                       // Ausgabe des Zeichens an die UART
      MAIN_P_UART1_TX_Read_Pointer++;                                // Lesezeiger erhhen
      MAIN_UART_TX_Buffer_Load--;
      
      // Ende des Arrays -> Ringpuffer!!
      if (MAIN_P_UART1_TX_Read_Pointer >= &MAIN_UART1_TX_Buffer[200]) 
        MAIN_P_UART1_TX_Read_Pointer = &MAIN_UART1_TX_Buffer[0];
    
}/* UART_TX_Interrupt() */





/* IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
 *
 * TIMERA0_Interrupt()
 *
 *
 *
 *
 */
#if __VER__ >= 300   
  #pragma vector = TIMERA0_VECTOR 
  __interrupt void TIMERA0_Interrupt(void)
#else
  interrupt [TIMERA0_VECTOR] void TIMERA0_Interrupt (void)
#endif  
{
  //RELOAD_SYSTIME();
  
  TACCR0 += MAIN_TIMER_PRESCALER_1MS; 
  //TOGGLE_LED_RX();
  _EINT();

  if (MAIN_Config_Timer)                  // Timer decrement
    MAIN_Config_Timer--;
  
  if (MAIN_Protocol_Timer)                  // Timer decrement
    MAIN_Protocol_Timer--;
}






/* --------------------------------------------------------------------------
 *
 * UART1_TX_WriteByte()
 *
 * writes byte to TX Buffer 
 *
 *
 */
uint16_t UART1_TX_WriteByte(uint8_t ucDataByte)
{
  if(MAIN_UART_TX_Buffer_Load > 200)
    return(0);
    
  *MAIN_P_UART1_TX_Write_Pointer = ucDataByte;					
  MAIN_P_UART1_TX_Write_Pointer++;								
  MAIN_UART_TX_Buffer_Load++;     
  // cyclic buffer check   
  if (MAIN_P_UART1_TX_Write_Pointer >= &MAIN_UART1_TX_Buffer[200])
      MAIN_P_UART1_TX_Write_Pointer = &MAIN_UART1_TX_Buffer[0]; 

  if(U1TCTL & TXEPT)
    IFG2 |= UTXIFG1;			// Force Interrupt
return(0);
}/* UART1_TX_WriteByte() */
